<?php

/*
  Plugin Name: Private Message
  Plugin URI: https://www.wpdownloadmanager.com/download/wordpress-private-message/
  Description: WordPress Private Message Plugin  allows site members to send private messages (PM) to each other or site admin.
  Author: W3 Eden, Inc.
  Version: 3.5.1
  Author URI: https://www.wpdownloadmanager.com/
  Text Domain: private-message
  Domain Path: /languages
 */

namespace PrivateMessage;

use PrivateMessage\__\__;
use PrivateMessage\Chat\ChatController;
use PrivateMessage\Facebook\FacebookController;
use PrivateMessage\Message\MessageController;
use PrivateMessage\Notice\NoticeController;
use PrivateMessage\Settings\SettingsController;
use PrivateMessage\User\UserController;

global $PrivateMessage;

/**
 * Text domain constant
 */
define('PM_TEXT_DOMAIN', 'private-message');

/**
 * Plugin admin cap
 */
if (!defined('PM_ADMIN_CAP'))
    define("PM_ADMIN_CAP", 'manage_options');

/**
 * Plugin dir name
 */
define("PM_DIR_NAME", basename(__DIR__));

/**
 * Plugin base dir
 */
define("PM_BASE_DIR", dirname(__FILE__) . '/');

/**
 * Plugin base url
 */
define("PM_BASE_URL", plugin_dir_url(__FILE__));

/**
 * Plugin asset url
 */
define("PM_ASSET_URL", plugin_dir_url(__FILE__) . 'assets/');

/**
 * Starter functions
 */
include __DIR__."/src/functions.php";

/**
 * Class PrivateMessage
 * @package PrivateMessage
 */
class PrivateMessage
{

    /**
     * @var array|false|string|string[]
     */
    public $clientDeviceID;
    /**
     * @var SettingsController
     */
    public $settings;
    /**
     * @var UserController
     */
    public $user;
    /**
     * @var MessageController
     */
    public $message;
    /**
     * @var ChatController
     */
    public $chat;
    /**
     * @var FacebookController
     */
    public $facebook;
    /**
     * @var NoticeController
     */
    public $notice;

    private static $wpdmpm_instance = null;

    /**
     * @return PrivateMessage|null
     */
    public static function instance()
    {

        //require_once dirname(__FILE__) . "/libs/class.PMAPI.php";

        if (is_null(self::$wpdmpm_instance)) {
            self::$wpdmpm_instance = new self();
        }
        return self::$wpdmpm_instance;
    }

    /**
     * PrivateMessage constructor.
     */
    function __construct()
    {
        $this->autoLoadClasses();
        $this->bsversion = get_option('__wpdm_bsversion', '');
        $ip = __::get_client_ip();
        $ip = str_replace(":", "__", $ip);
        $ip = str_replace(".", "_", $ip);
        $this->clientDeviceID = $ip;

        $this->settings = SettingsController::getInstance();
        $this->user = UserController::getInstance();
        $this->message = MessageController::getInstance();
        $this->chat = ChatController::getInstance();
        $this->facebook = FacebookController::getInstance();
        $this->notice = NoticeController::getInstance();

        add_action("wp_head", array($this, 'scripts'), 999999);
        add_action("admin_head", [$this, 'adminHead']);

        add_action('wp_enqueue_scripts', array($this, 'enqueueScripts'));
        add_action('admin_enqueue_scripts', array($this, 'adminEnqueueScripts'));

        add_action("init", array($this, "registerPostType"));

        add_filter("wpdm_email_templates", array($this, 'emailTemplate'));


        add_action("admin_head", array($this, 'scripts'), 0);
    }

    /**
     * Class autoloader
     */
    function autoLoadClasses()
    {
        spl_autoload_register(function ($class) {

            $prefix = 'PrivateMessage\\';
            $base_dir = __DIR__ . '/src/';

            $len = strlen($prefix);
            if (strncmp($prefix, $class, $len) !== 0) {
                return;
            }

            $relative_class = substr($class, $len);
            $file = $base_dir . str_replace('\\', '/', $relative_class) . '.php';

            if (file_exists($file)) {
                require $file;
            }
        });
    }

    function scripts()
    {
        ?>
        <script>
            var wpdmpmAPI = <?php echo json_encode(array(
                'root' => esc_url_raw(rest_url()),

                'chat_id' => esc_url_raw(rest_url('wpdmpm/v1/get-chat-id')),
                'send_chat_msg' => esc_url_raw(rest_url('wpdmpm/v1/send-chat-msg')),
                'chat_history' => esc_url_raw(rest_url('wpdmpm/v1/connect-admin')),

                'send_message' => esc_url_raw(rest_url('wpdmpm/v1/send-message')),
                'send_message_admin' => esc_url_raw(rest_url('wpdmpm/v1/send-message-admin')),
                'create_reply' => esc_url_raw(rest_url('wpdmpm/v1/create-reply')),
                'get_messages' => esc_url_raw(rest_url('wpdmpm/v1/get-messages')),
                'get_message' => esc_url_raw(rest_url('wpdmpm/v1/get-message')),
                'delete_message' => esc_url_raw(rest_url('wpdmpm/v1/delete-message')),
                'delete_message_admin' => esc_url_raw(rest_url('wpdmpm/v1/delete-message-admin')),
                'check_new_message' => esc_url_raw(rest_url('wpdmpm/v1/check-new-message')),
                'get_link_meta' => esc_url_raw(rest_url('wpdmpm/v1/get-link-meta')),

                'users' => esc_url_raw(rest_url('wpdmpm/v1/users')),
                'user_profile' => esc_url_raw(rest_url('wpdmpm/v1/user-profile')),
                'follow_user' => esc_url_raw(rest_url('wpdmpm/v1/follow-user')),
                'unfollow_user' => esc_url_raw(rest_url('wpdmpm/v1/unfollow-user')),
                'add_contact' => esc_url_raw(rest_url('wpdmpm/v1/add-contact')),
                'remove_contact' => esc_url_raw(rest_url('wpdmpm/v1/remove-contact')),
                'block_contact' => esc_url_raw(rest_url('wpdmpm/v1/block-contact')),
                'get_followers' => esc_url_raw(rest_url('wpdmpm/v1/get-followers')),
                'get_followings' => esc_url_raw(rest_url('wpdmpm/v1/get-followings')),

                'asset_url' => PM_ASSET_URL,
                'home_url' => home_url('/'),
                'ajax_url' => admin_url('/admin-ajax.php'),
                'nonce' => wp_create_nonce('wp_rest'),
                'admin_avatar' => $this->settings->general->admin_avatar,
                'visitor_avatar' => $this->settings->general->visitor_avatar,
                'multiline_msg' => $this->settings->general->multiline_msg,
                'chatbox_width' => $this->settings->general->chatbox_size['width'],
                'chatbox_height' => $this->settings->general->chatbox_size['height']
            )); ?>;
            /*wpdmpmAPI.request = function (endpoint, data)*/
        </script>
        <?php
    }

    /**
     * Plugin icon
     */
    function adminHead()
    {
        $this->scripts();
        ?>
        <style>
            #adminmenu #toplevel_page_wpdmpm .menu-icon-generic div.wp-menu-image:before {
                content: '';
                background: url("<?= PM_ASSET_URL ?>/images/pmsg.svg") center center no-repeat;
                background-size: contain;
                filter: grayscale(100%);
            }

            #adminmenu #toplevel_page_wpdmpm.wp-menu-open .menu-icon-generic div.wp-menu-image:before,
            #adminmenu #toplevel_page_wpdmpm.opensub .menu-icon-generic div.wp-menu-image:before {
                filter: grayscale(0%);
            }
        </style>
        <?php
    }

    /**
     * Load Plugin Text Domain
     */
    function loadTextdomain()
    {
        load_plugin_textdomain(PM_TEXT_DOMAIN, WP_PLUGIN_URL . "/private-message/languages/", 'private-message/languages/');
    }

    function emailTemplate($templates)
    {
        $domain = str_replace("www.", "", $_SERVER['HTTP_HOST']);
        $noreply = "noreply@{$domain}";
        $templates['private-message'] = array(
            'label' => __('Private Message', 'wpdmpro'),
            'for' => 'Receiver',
            'default' => array('subject' => '[#subject#]',
                'from_name' => get_option('blogname'),
                'from_email' => $noreply,
                'message' => '[#message#]</b><br/><br/>Best Regards,<br/>Support Team<br/><b><a href="[#homeurl#]">[#sitename#]</a></b>'
            )
        );


        return $templates;
    }

    /**
     * Enqueue front-end scripts
     */
    function enqueueScripts()
    {
        global $post;
        $pm_active = false;
        $showHere = true;
        $showHere = apply_filters("PrivateMessage/enqueueScripts", $showHere);
        if ($post && (has_shortcode($post->post_content, 'private_message')
                || has_shortcode($post->post_content, 'pm_to_user')
                || has_shortcode($post->post_content, 'pm_to_author')
                || has_shortcode($post->post_content, 'woocommerce_my_account')
                || __::query_var('udb_page') === 'messages'
                || $this->settings->general->pm_page_id === $post->ID
                || $showHere
            )
        ) {
            wp_enqueue_script('jquery');
            wp_enqueue_script('jquery-form', array('jquery'));
            wp_enqueue_script('jquery-ui-core', array('jquery'));
            wp_enqueue_script('jquery-ui-autocomplete', array('jquery'));
            wp_enqueue_script('media-upload');
            if(!get_option('__pmcssfix', false))
                wp_enqueue_style('wppm-bootstrap',  PM_ASSET_URL.'bootstrap/css/bootstrap.min.css', 999);
            if(!get_option('__pmjsfix', false))
                wp_enqueue_script('wpdm-bootstrap', PM_ASSET_URL.'bootstrap/js/bootstrap.bundle.js', array('jquery'));
            if(!get_option('__pmfafix', false))
                wp_enqueue_style('wpdm-font-awesome', 'https://use.fontawesome.com/releases/v5.12.1/css/all.css', 999);
            wp_enqueue_script('media-upload');
            wp_enqueue_media();
            $pm_active = true;
        }
        if ($pm_active || $this->settings->general->chat_active || $this->notice->count() > 0) {
            wp_enqueue_script("wppmsg-vue", 'https://cdn.jsdelivr.net/npm/vue');
        }
        if ($pm_active || $this->settings->general->chat_active) {
            wp_enqueue_style('wppmsg', PM_ASSET_URL . 'css/style.min.css');
            wp_enqueue_script("wppmsg", PM_ASSET_URL . 'js/wppmsg.js', array('jquery'));
        }

    }

    /**
     * Enqueue admin scripts
     * @param $hook
     */
    function adminEnqueueScripts($hook)
    {
        if ($hook === 'post.php' || $hook === 'post-new.php' || $hook === 'edit.php') $hook = __::query_var('post_type') ? __::query_var('post_type') : get_post_type();
        if (in_array($hook, ['toplevel_page_wpdmpm', 'messages_page_wpdmchat', 'messages_page_pmsettings', 'notice'])) {
            wp_dequeue_style("wpdm-admin-bootstrap");
            wp_dequeue_script("wpdm-admin-bootstrap");
            wp_enqueue_script('jquery-form');
            wp_enqueue_style('wppm-bootstrap', PM_ASSET_URL . 'bootstrap/css/bootstrap.min.css', 999);
            wp_enqueue_style('wpdmpm-style', PM_ASSET_URL . 'css/admin-style.min.css');
            wp_enqueue_style('wppmsg-chosen-style', PM_ASSET_URL . 'chosen-select/chosen.min.css');
            wp_enqueue_style('wpdm-font-awesome', 'https://use.fontawesome.com/releases/v5.12.1/css/all.css', 999);
            wp_enqueue_script('wpdm-front-bootstrap', PM_ASSET_URL . 'bootstrap/js/bootstrap.bundle.js', array('jquery'));
            wp_enqueue_script('wppmsg-chosen-js', PM_ASSET_URL . 'chosen-select/chosen.jquery.min.js');
            wp_enqueue_script("wpdmpm-vue", 'https://cdn.jsdelivr.net/npm/vue');
            wp_enqueue_script('wppmsg-admin', PM_ASSET_URL . 'js/wppmsg-admin.js', array('jquery'));
            wp_enqueue_script('media-upload');
            wp_enqueue_media();
        }

    }


    /**
     * Register message post type
     */
    function registerPostType()
    {
        $labels = array(
            'name' => 'Messages',
            'singular_name' => 'Message',
            'add_new' => 'Add New',
            'add_new_item' => 'Add New Message',
            'edit_item' => 'Edit Message',
            'new_item' => 'New Message',
            'all_items' => 'All Messages',
            'view_item' => 'View Message',
            'search_items' => 'Search Messages',
            'not_found' => 'No Messages found',
            'not_found_in_trash' => 'No Messages found in Trash',
            'parent_item_colon' => '',
            'menu_name' => 'Messages'
        );

        $args = array(
            'labels' => $labels,
            'public' => false,
            'publicly_queryable' => false,
            'show_ui' => true,
            'show_in_menu' => false,
            'query_var' => true,
            'rewrite' => array('slug' => 'message'),
            'capability_type' => 'post',
            'has_archive' => false,
            'hierarchical' => false,
            'menu_position' => null,
            'menu_icon' => "dashicons-format-chat",
            'supports' => array('title', 'editor', 'author', 'comments')
        );

        register_post_type('message', $args);

        $labels = array(
            'name' => 'Notice',
            'singular_name' => 'Notice',
            'add_new' => 'Add New',
            'add_new_item' => 'Add New Notice',
            'edit_item' => 'Edit Notice',
            'new_item' => 'New Notice',
            'all_items' => 'All Notices',
            'view_item' => 'View Status',
            'search_items' => 'Search Notices',
            'not_found' => 'No Notices found',
            'not_found_in_trash' => 'No Notices found in Trash',
            'parent_item_colon' => '',
            'menu_name' => 'Notice'
        );

        $args = array(
            'labels' => $labels,
            'public' => false,
            'publicly_queryable' => false,
            'show_ui' => true,
            'show_in_menu' => false,
            'query_var' => true,
            'rewrite' => array('slug' => 'notice'),
            'capability_type' => 'post',
            'has_archive' => false,
            'hierarchical' => false,
            'menu_position' => null,
            'menu_icon' => "dashicons-format-chat",
            'supports' => array('title', 'editor')
        );

        register_post_type('notice', $args);
    }


}

$PrivateMessage = PrivateMessage::instance();



