<?php


namespace PrivateMessage\Chat;

if (!defined("ABSPATH")) die("Shit happens!");

use PrivateMessage\__\__;
use PrivateMessage\__\__Session;
use PrivateMessage\__\__Template;
use PrivateMessage\PrivateMessage;


class ChatController
{

    private static $instance;

    public static function getInstance()
    {
        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    private function __construct()
    {
        new RestAPI();

        add_action("init", [$this, "chatWindowClean"]);
        add_action("init", [$this, "chatBoxFrame"]);
        add_action("admin_menu", [$this, "adminMenu"]);
        add_action('wp_ajax_wpdm_active_chat_users', [$this, 'activeChatUsers']);
        add_filter('wp_footer', [$this, 'chatButton']);

        add_shortcode("live_chat", [$this, "chatBox"]);
        add_shortcode("live_chat_button", [$this, "chatPopupButton"]);

    }

    function adminMenu()
    {
        add_submenu_page("wpdmpm", __('Live Chat', PM_TEXT_DOMAIN), __('Private Chat', PM_TEXT_DOMAIN), PM_ADMIN_CAP, 'wpdmchat', [$this, 'chatAdmin']);
    }

    function allowedToChat()
    {
        $roles = wp_get_current_user()->roles;
        $roles[] = 'guest';
        $allowed_roles = PrivateMessage()->settings->general->chat_allowed_roles;
        $is_allowed = count(array_intersect($roles, $allowed_roles)) ? true : false;
        return $is_allowed;
    }

    function activeChatUsers()
    {
        $chat_dir = self::getChatDir();
        $active_users = scandir($chat_dir);
        $chat_users = $new_update = array();
        foreach ($active_users as $active_user) {
            if (strstr($active_user, ".json")) {
                $chat_id = str_replace(".json", "", $active_user);
                if ($chat_id !== '') {
                    $chat_file = $chat_dir . $active_user;
                    $chat_data = file_get_contents($chat_file);
                    $chat_data = json_decode($chat_data);
                    $users = (array)$chat_data->users;
                    if (count($users) > 0) {
                        if (count($users) < 3) {
                            $_ips = array_keys($users);
                            $ip = str_replace("__", ":", $_ips[0]);
                            $ip = str_replace("_", ".", $ip);
                            $user = $users[$_ips[0]];
                            $avatar = "https://www.gravatar.com/avatar/" . md5($user);
                            if (email_exists($user)) {
                                $user = get_user_by('email', $user);
                                $user = $user->display_name;
                            }
                        } else {
                            $user = "#" . $chat_id;
                            $avatar = "https://cdn4.iconfinder.com/data/icons/web-ui-color/128/Chat-512.png";
                        }
                        $new = isset($chat_data->new) ? $chat_data->new : 0;
                        $status = is_object($chat_data) && isset($chat_data->last_msg) && ((time() - (int)$chat_data->last_msg) < 300) ? 'active' : 'inactive';
                        $lastm = isset($chat_data->last_msg) ? date(get_option('date_format') . " " . get_option('time_format'), $chat_data->last_msg) : "--------";
                        $chat_users[] = array('user' => $user, 'avatar' => $avatar, 'ip' => $ip, 'channel' => $chat_id, 'status' => $status, 'new' => $new, 'last_active' => $lastm);
                    }
                }

            }
        }

        $chat_users = array_reverse($chat_users);
        wp_send_json($chat_users);

    }

    public static function getChatDir($url = false)
    {
        $_upload_dir = wp_upload_dir();
        $chat_dir = $_upload_dir['basedir'] . "/wpdmpm-chats/";
        $chat_dir_url = $_upload_dir['baseurl'] . "/wpdmpm-chats/";
        if (!file_exists($chat_dir))
            @mkdir($chat_dir, 0755);
        return $url ? $chat_dir_url : $chat_dir;
    }

    public static function getChatFile($chat_id = null, $url = false)
    {
        $_upload_dir = wp_upload_dir();
        $chat_dir = $_upload_dir['basedir'] . "/wpdmpm-chats/";
        $chat_dir_url = $_upload_dir['baseurl'] . "/wpdmpm-chats/";
        $chat_id = $chat_id == null ? __Session::get('chat_id') : $chat_id;
        $chat_file = $chat_dir . "{$chat_id}.json";
        $chat_file_url = $chat_dir_url . "{$chat_id}.json";
        if (!file_exists($chat_file)) {
            $data = new \stdClass;
            $data->users = array();
            $data->data = array();
            file_put_contents($chat_file, json_encode($data));
        }
        return $url ? $chat_file_url : $chat_file;
    }

    public static function getChatData($chat_id)
    {
        $chat_file = self::getChatFile($chat_id);
        $data = file_get_contents($chat_file);
        $data = json_decode($data);
        if (!is_object($data)) {
            $data = new \stdClass;
            $data->users = array();
            $data->data = array();
        }
        $data->users = (array)$data->users;
        $data->data = (array)$data->data;
        return $data;
    }

    public static function saveChatData($chat_id, $data)
    {
        $chat_file = self::getChatFile($chat_id);
        file_put_contents($chat_file, json_encode($data));
    }

    public static function killChat($chat_id)
    {
        $_upload_dir = wp_upload_dir();
        $chat_dir = $_upload_dir['basedir'] . "/wpdmpm-chats/";
        $chat_dir_url = $_upload_dir['baseurl'] . "/wpdmpm-chats/";
        $chat_id = $chat_id == null ? __Session::get('chat_id') : $chat_id;
        $chat_file = $chat_dir . "{$chat_id}.json";
        __Session::clear('chat_id');
        if (file_exists($chat_file))
            @unlink($chat_file);
    }

    function chatBox()
    {
        global $current_user;

        ob_start();
        include __Template::locate("chatbox/embed.php", __DIR__ . '/views');

        ?>

        <script>
            var $ = jQuery;
            var chat_path = '<?php echo self::getChatDir(true) ?>';
            var greet = {
                sender_pic: wpdmpmAPI.admin_avatar,
                sender_name: 'Admin',
                body: '<?php echo is_user_logged_in() ? 'Hello ' . $current_user->display_name . ", welcome back. How may I help you today?" : 'Please give me your email address in case we disconnect in the middle somehow'; ?>'
            };
            var __chat = new Vue({
                el: '#__chat_wpdmpm',
                data: {
                    messages: [
                        greet
                    ]
                }
            });
        </script>
        <?php
        return ob_get_clean();
    }

    function chatBoxFrame()
    {
        if (__::query_var('wpdmchatbox', 'int') === 1) {
            include __DIR__ . '/views/live-chat-container.php';
            die();
        }
    }

    function chatWindowClean()
    {

        if (substr_count($_SERVER['REQUEST_URI'], '/w3echat')) {
            include __DIR__ . '/views/chat-embed-clean.php';
            die();
        }
    }

    function chatButton()
    {
//        disable private chat if facebook chat is active
        if (get_option('__chatactivefb') !== '') return;
        global $current_user, $post;
        if (
            (is_object($post) && has_shortcode($post->post_content, 'live_chat')) ||
            substr_count($_SERVER['REQUEST_URI'], '/w3echat') ||
            !(int)PrivateMessage()->settings->general->chat_active
        ) return;

        //if ((int)get_option('__chatactive', 0) !== 1) return;
        $left_or_right = \PrivateMessage()->settings->general->chatbox_pos['c'];
        $hspace = \PrivateMessage()->settings->general->chatbox_pos['h'];
        $vspace = \PrivateMessage()->settings->general->chatbox_pos['v'];
        $poscss = "{$left_or_right}: {$hspace}px !important;bottom: {$vspace}px !important";

        ?>

        <script>
            jQuery(function ($) {
                $('body').append("<iframe id='wpdmchatframe' src='<?php echo home_url('/?wpdmchatbox=1'); if (__::query_var('wpdmchatid') !== '') echo "&wpdmchatid=" . __::query_var('wpdmchatid'); ?>' style='max-height:100%;max-width:100%;position:fixed !important;z-index: 9999 !important;<?= $poscss; ?>;border:0 !important;box-shadow:none !important;width: 72px;height: 72px;background: transparent !important;'></iframe>");
            });

            function wpdmchatframeaw(w, h) {
                jQuery('#wpdmchatframe').css('width', w + 'px').css('height', h + 'px');
            }

            var $ = jQuery;
            var chat_path = '<?php echo ChatController::getChatDir(true) ?>';
            var greet = {
                sender_pic: wpdmpmAPI.admin_avatar,
                sender_name: 'Admin',
                body: '<?php echo is_user_logged_in() ? 'Hello ' . $current_user->display_name . ", welcome back. How may I help you today?" : 'Please give me your email address in case we disconnect in the middle somehow'; ?>'
            };
            var __chat = new Vue({
                el: '#__chat_wpdmpm',
                data: {
                    messages: [
                        greet
                    ]
                }
            });
        </script>
        <?php
    }

    function chatPopupButton($params = [])
    {
        $class = __::valueof($params, 'class', 'btn btn-info');
        $label = __::valueof($params, 'label', 'Live Chat');
        $url = home_url('w3echat');
        return "<div class='w3eden' style='display: inline-block'><button type='button' onclick=\"WPPMSG.popupWindow('{$url}', 'Live Chat', 400, 550)\" class='{$class}'>{$label}</button></div>";
    }

    function chatAdmin()
    {
        include __DIR__ . '/views/chat-admin.php';
    }

}
