<?php


namespace PrivateMessage\Chat;


use PrivateMessage\__\__;
use PrivateMessage\__\__Crypt;
use PrivateMessage\__\__Email;
use PrivateMessage\__\__MailUI;
use PrivateMessage\__\__Session;
use PrivateMessage\Dialogflow\Dialogflow;


class RestAPI
{
    function __construct()
    {
        add_action( 'rest_api_init', [$this, 'apiInit'] );
    }

    function apiInit(){


        register_rest_route( 'wpdmpm/v1', '/get-chat-id', array(
            'methods' => 'GET',
            'callback' => array($this, 'getChatID'),
            'permission_callback' => '__return_true'
        ) );

        register_rest_route( 'wpdmpm/v1', '/connect-admin', array(
            'methods' => 'GET',
            'callback' => array($this, 'connectAdmin'),
            'permission_callback' => '__return_true'
        ) );

        register_rest_route( 'wpdmpm/v1', '/send-chat-msg', array(
            'methods' => 'POST',
            'callback' => array($this, 'sendChatMessage'),
            'permission_callback' => '__return_true'
        ) );

    }

    function getChatID(){
        global $current_user;
        $chat_id = __Session::get('chat_id');
        if(!$chat_id)
            $chat_id = "WPPMSG".strtoupper(uniqid());
        ChatController::getChatFile($chat_id);
        __Session::set("chat_id", $chat_id, 18000);
        wp_send_json(array('chat_id' => $chat_id, 'success' => true));
    }

    function connectAdmin(){
        //todo: need auth check
        global $current_user;
        $chat_id = __::query_var('wpdmpm_chat_id');
        if(__::query_var('sender') === 'admin'){

        }
        $chat_file = ChatController::getChatFile($chat_id);
        $chat = json_decode(file_get_contents($chat_file));
        if(!$chat) wp_send_json($chat);;
        if(__::query_var('sender') === 'admin' && current_user_can(PM_ADMIN_CAP) && !isset($chat->users->admin)){
            $chat->users = (array)$chat->users;
            $chat->users['admin'] = $current_user->user_email;
            ChatController::saveChatData($chat_id, $chat);
        }

        $users = (array)$chat->users;
        foreach ($users as $id => &$user){
            $_user = get_user_by( 'email', $user );
            if($_user) $user = $_user->data;
        }

        wp_send_json($chat);
    }

    function sendChatMessage(){
        global $PrivateMessage, $current_user;

        $message = __::query_var('message', 'escs');

        if($message === '/x'){
            ChatController::killChat(__::query_var('wpdmpm_chat_id'));
            wp_send_json(array('success' => true));
        }

        $chat = ChatController::getChatData(__::query_var('wpdmpm_chat_id'));

        $sender = __::query_var('sender') === 'admin' && current_user_can(PM_ADMIN_CAP) ? 'admin' : $PrivateMessage->clientDeviceID;
        $mailed = 0;
        $_message = '';
        $anonymous = 0;

        $url_regex = '/\b(?:(?:https?|ftp):\/\/|www\.)[-a-z0-9+&@#\/%?=~_|!:,.;]*[-a-z0-9+&@#\/%=~_|]/i';

        if(!isset($chat->users[$sender]) || __::query_var('action') === 'join'){

            if(!is_user_logged_in() && !is_email($message) && PrivateMessage()->settings->general->anonymous_chat) {
                $domain = parse_url(home_url());
                $domain = __::valueof($domain, 'host');
                $domain = $domain !== 'localhost' ? : 'w3eden.com';
                $_message = $message;
                $anonymous = 1;
                $message = __::query_var('wpdmpm_chat_id')."@{$domain}";
            }
            //Ask for email when user is not logged-in or email is not given already
            if(!is_user_logged_in() && !is_email($message)) {
                wp_send_json(array('success' => false, 'reply' => 'Please enter a valid email address'));
            } else {
                //User already sent his/her email, as for the query
                if(!is_user_logged_in()) {
                    $chat->users[$PrivateMessage->clientDeviceID] = $message;
                    $avatar = is_email($message) && !$anonymous ? 'https://www.gravatar.com/avatar/'.md5($message) : PrivateMessage()->settings->general->visitor_avatar;
                    $reply = __('Thank you! Please let me know your query', PM_TEXT_DOMAIN);
                    $name  = explode("@", $message);
                    $name  = $name[0];

                    preg_match_all($url_regex, $message, $urls);
                    foreach ($urls[0] as $url) {
                        $effective_url = __::getEffectiveUrl($url);
                        $new_url = '<a href="' . $effective_url . '" target="_blank">' . $url . '</a>';
                        $message = str_replace($url, $new_url, $message);
                    }

                    $msg = array(
                        'sender'          => $PrivateMessage->clientDeviceID,
                        'sender_name'     => $name,
                        'sender_pic'      => $avatar,
                        'type'            => 'user',
                        'urls'            => $urls[0],
                        'body'            => wp_kses_post($message),
                        'time'            => time()
                    );
                    $chat->data[] = $msg;
                    if($anonymous) {
                        $message = $_message;
                        goto CONTINUECHAT;
                    }
                    preg_match_all($url_regex, $message, $urls);
                    foreach ($urls[0] as $url) {
                        $effective_url = __::getEffectiveUrl($url);
                        $new_url = '<a href="' . $effective_url . '" target="_blank">' . $url . '</a>';
                        $message = str_replace($url, $new_url, $message);
                    }
                    $msg = array(
                        'sender'          => 'system',
                        'sender_name'     => 'System',
                        'sender_pic'      => PrivateMessage()->settings->general->admin_avatar,
                        'type'            => 'admin',
                        'urls'            => $urls[0],
                        'body'            => wp_kses_post($message),
                        'time'            => time()
                    );
                    $chat->data[] = $msg;
                    $chat->last_msg = time();
                    $chat->new = 1;
                    ChatController::saveChatData(__::query_var('wpdmpm_chat_id'), $chat);

                    $admin_avatar = isset($chat->users->admin) ? 'https://www.gravatar.com/avatar/'.md5($chat->users->admin) : 'https://www.gravatar.com/avatar/'.md5(get_option('admin_email'));
                    wp_send_json(array('success' => true, 'reply'=> $reply, 'avatar' => $avatar, 'admin_avatar' => $admin_avatar, 'user' => __Crypt::encrypt($PrivateMessage->clientDeviceID)));
                }
                //User sent his query
                else {
                    $chat->users[$PrivateMessage->clientDeviceID] = $current_user->user_email;
                    $avatar = is_user_logged_in() ? 'https://www.gravatar.com/avatar/'.md5($current_user->user_email) : PrivateMessage()->settings->general->admin_avatar;

                    preg_match_all($url_regex, $message, $urls);
                    foreach ($urls[0] as $url) {
                        $effective_url = __::getEffectiveUrl($url);
                        $new_url = '<a href="' . $effective_url . '" target="_blank">' . $url . '</a>';
                        $message = str_replace($url, $new_url, $message);
                    }

                    $msg = array(
                        'sender'          => $sender,
                        'sender_name'          => $current_user->display_name,
                        'sender_pic'          => 'https://www.gravatar.com/avatar/'.md5($current_user->user_email),
                        'type'            => $sender === 'admin' ? 'admin' : 'user',
                        'urls'            => $urls[0],
                        'body'            => wp_kses_post($message),
                        'time'            => time()
                    );
                    $chat->data[] = $msg;
                    if ($msg['type'] === 'user'){
                        $chat->last_msg = time();
                        $chat->new = 1;
                        if(PrivateMessage()->settings->chatautomation->chat_bot_active){
                            $dialogflow = new Dialogflow();
                            $reply = $dialogflow->getReply(stripslashes_deep($message));
                            $msg = array(
                                'sender'          => 'system',
                                'sender_name'     => 'System',
                                'sender_pic'      => PrivateMessage()->settings->general->admin_avatar,
                                'type'            => 'admin',
                                'urls'            => $urls[0],
                                'body'            => wp_kses_post($message),
                                'time'            => time()
                            );
                            $chat->data[] = $msg;
                            $chat->last_msg = time();
                        }

                    } else {
                        $chat->new = 0;
                        //Check if user is online/active
                        $active = isset($chat->last_msg) && ((time() - (int)$chat->last_msg) < 30) ? true : false;
                        //Send email if user is not online
                        if($active === false && PrivateMessage()->settings->email->chat_mail_to_user){
                            $users = array_values((array)$chat->users);
                            $url = (int)PrivateMessage()->settings->general->chat_page_id > 0 ? add_query_arg(['wpdmchatid' => __::query_var('wpdmpm_chat_id')], get_permalink((int)PrivateMessage()->settings->general->chat_page_id)) : home_url("?wpdmchatid=".__::query_var('wpdmpm_chat_id'));
                            $params = array(
                                'to_email'      => $users[0],
                                'subject'       => __("You got a reply", PM_TEXT_DOMAIN),
                                'reply_url'       => $url,
                                'message'       => "<br/>".__MailUI::panel(__("Message:", PM_TEXT_DOMAIN), array( stripslashes_deep($message)), "<a style='margin: 0 !important;display: block;text-align:center;border-radius: 2px' class='button button-green' href='{$url}'>".__('Reply', PM_TEXT_DOMAIN)."</a>")
                            );
                            __Email::send('chat', $params);
                            $mailed = 1;
                        }
                    }
                    ChatController::saveChatData(__::query_var('wpdmpm_chat_id'), $chat);
                    $admin_avatar = isset($chat->users->admin) ? 'https://www.gravatar.com/avatar/'.md5($chat->users->admin) : 'https://www.gravatar.com/avatar/'.md5(get_option('admin_email'));
                    wp_send_json(array('success' => true, 'avatar' => $avatar, 'admin_avatar' => $admin_avatar, 'mailed' => $mailed));
                }
            }
        } else {
            CONTINUECHAT:

            $email = $chat->users[$sender];
            $user = get_user_by('email', $email);
            $name  = explode("@", $email);
            $name  = !$user ? $name[0] : $user->display_name;

            preg_match_all($url_regex, $message, $urls);
            foreach ($urls[0] as $url) {
                $effective_url = __::getEffectiveUrl($url);
                $new_url = '<a href="' . $effective_url . '" target="_blank">' . $url . '</a>';
                $message = str_replace($url, $new_url, $message);
            }

            $msg = array(
                'sender'          => $sender,
                'sender_name'     => $name,
                'sender_pic'      => 'https://www.gravatar.com/avatar/'.md5($email),
                'type'            => $sender === 'admin' ? 'admin' : 'user',
                'urls'            => $urls[0],
                'body'            => wp_kses_post($message),
                'time'            => time()
            );

            $chat->data[] = $msg;

            if ($msg['type'] === 'user'){
                $chat->last_msg = time();
                $chat->new = 1;

                if(PrivateMessage()->settings->chatautomation->chat_bot_active){
                    $dialogflow = new Dialogflow();
                    $reply = $dialogflow->getReply(stripslashes_deep($message));
                    $msg = array(
                        'sender'          => 'system',
                        'sender_name'     => 'System',
                        'sender_pic'      => PrivateMessage()->settings->general->admin_avatar,
                        'type'            => 'admin',
                        'urls'            => $urls[0],
                        'body'            => wp_kses_post($message),
                        'time'            => time()
                    );
                    $chat->data[] = $msg;
                    $chat->last_msg = time();
                }

            } else {
                $chat->new = 0;
                $active = isset($chat->last_msg) && ((time() - (int)$chat->last_msg) < 30) ? true : false;
                if($active === false && PrivateMessage()->settings->email->chat_mail_to_user){
                    $users = array_values((array)$chat->users);
                    $url = (int)PrivateMessage()->settings->general->chat_page_id > 0 ? add_query_arg(['wpdmchatid' => __::query_var('wpdmpm_chat_id')], get_permalink((int)PrivateMessage()->settings->general->chat_page_id)) : home_url("?wpdmchatid=".__::query_var('wpdmpm_chat_id'));
                    $params = array(
                        'to_email'      => $users[0],
                        'subject'       => __("You got a reply", PM_TEXT_DOMAIN),
                        'reply_url'       => $url,
                        'message'       => "<br/>".__MailUI::panel(__("Message:", PM_TEXT_DOMAIN), array( stripslashes_deep($message)), "<a style='margin: 0 !important;display: block;text-align:center;border-radius: 2px' class='button button-green' href='{$url}'>Reply</a>")
                    );
                    __Email::send('chat', $params);
                    $mailed = 1;
                }
            }
            ChatController::saveChatData(__::query_var('wpdmpm_chat_id'), $chat);
        }
        wp_send_json(array('success' => true, 'data' => $chat->data, 'mailed' => $mailed));
    }
}
