<?php

namespace PrivateMessage\Facebook;
class Options
{

    private static $instance;

    public static function getInstance()
    {
        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    public function __construct()
    {
        add_action('admin_enqueue_scripts', [$this, 'addStyles']);
        add_action('admin_enqueue_scripts', [$this, 'localizeAjax']);
        add_action('wp_ajax_wppm_fbc_update_options', [$this, 'updateOptions']);
    }


    function updateOptions()
    {

        if (current_user_can('manage_options')) {
            check_ajax_referer('update_fmcc_code');
            update_option('wppm_fbc_pageID', $this->sanitizePageID($_POST['pageID']));
            update_option('wppm_fbc_locale', $this->sanitizeLocale($_POST['locale']));
        }
        wp_die();
    }

    function sanitizePageID($input)
    {
        if (preg_match('/^\d+$/', $input)) {
            return $input;
        } else {
            return '';
        }
    }

    function sanitizeLocale($input)
    {
        if (preg_match('/^[A-Za-z_]{4,5}$/', $input)) {
            return $input;
        } else {
            return '';
        }
    }

    function addStyles()
    {
        wp_register_script(
            'launch_script',
            plugins_url('/script.js?2', __FILE__),
            array('jquery')
        );
        wp_enqueue_script('launch_script');

        wp_enqueue_style(
            'wppm-fbcadmin-styles',
            plugins_url('/settings.css', __FILE__),
            false,
            '1.0',
            'all'
        );
    }

    function localizeAjax()
    {
        if (current_user_can('manage_options')) {
            $ajax_object = array(
                'nonce' => wp_create_nonce('update_fmcc_code')
            );

            wp_register_script('code_script',
                plugin_dir_url(__FILE__) . 'script.js');
            wp_localize_script('code_script', 'ajax_object', $ajax_object);
            wp_enqueue_script('code_script');
        }

    }
}
