<?php


namespace PrivateMessage\Notice;


use PrivateMessage\__\__;
use PrivateMessage\__\__Crypt;
use PrivateMessage\__\__MailUI;
use PrivateMessage\__\__Email;


class RestAPI
{
    function __construct()
    {

        $pms = maybe_unserialize(get_option('pm_settings'));
        if(is_array($pms)) {
            foreach ($pms as $key => $value) {
                $this->$key = $value;
            }

        }

        add_action( 'rest_api_init', array($this, 'apiInit'));
    }


    function apiInit(){

        register_rest_route( 'wpdmpm/v1', '/send-message', array(
            'methods' => 'POST',
            'callback' => array($this, 'sendMessage'),
            'permission_callback' => '__return_true'
        ) );

        register_rest_route( 'wpdmpm/v1', '/send-message-admin', array(
            'methods' => 'POST',
            'callback' => array($this, 'sendMessageAdmin'),
            'permission_callback' => function() { return current_user_can(PM_ADMIN_CAP); }
        ) );

        register_rest_route( 'wpdmpm/v1', '/create-reply', array(
            'methods' => 'POST',
            'callback' => array($this, 'createReply'),
            'permission_callback' => '__return_true'
        ) );


        register_rest_route( 'wpdmpm/v1', '/get-messages', array(
            'methods' => 'GET',
            'callback' => array($this, 'getMessages'),
            'permission_callback' => '__return_true'
        ) );


        register_rest_route( 'wpdmpm/v1', '/get-message', array(
            'methods' => 'GET',
            'callback' => array($this, 'getMessage'),
            'permission_callback' => '__return_true'
        ) );

        register_rest_route( 'wpdmpm/v1', '/delete-message', array(
            'methods' => 'POST',
            'callback' => array($this, 'deleteMessage'),
            'permission_callback' => function() { return current_user_can(PM_ADMIN_CAP); }
        ) );

        register_rest_route( 'wpdmpm/v1', '/check-new-message', array(
            'methods' => 'GET',
            'callback' => array($this, 'checkNewMessage'),
            'permission_callback' => '__return_true'
        ) );

        register_rest_route( 'wpdmpm/v1', '/users', array(
            'methods' => 'GET',
            'callback' => array($this, 'searchUsers'),
            'permission_callback' => function() { return current_user_can(PM_ADMIN_CAP); }
        ) );

    }

    function checkNewMessage(){
        global $current_user, $wpdb;

        if(!is_user_logged_in()) wp_send_json(array('error' => true, 'message' => 'Authorization Failed!'));
        $args = array(
            'post_type' => 'message',
            'posts_per_page' => -1,
            'meta_query' => array(
                'relation' => 'AND',
                array(
                    'key' => '__wpdmpm_inbox_'.$current_user->ID,
                    'value' => 1
                ),

                array(
                    'key' => '__deleted_by_receiver',
                    'value' => 0
                )
            )
        );

        $messages = new \WP_Query($args);
        $new_messages = $messages->post_count;
        $oldcount = (int)get_user_meta($current_user->ID, "__wpdmpm_inbox_message_count", true);
        if($new_messages !== $oldcount){
            update_user_meta($current_user->ID, "__wpdmpm_inbox_message_count", $new_messages);
            if($new_messages !== $oldcount)
                wp_send_json(array('new_message' => $new_messages - $oldcount));
        }

        $key = "__msg_status_{$current_user->ID}";
        $unread = (int)$wpdb->get_var("select count(*) from {$wpdb->prefix}postmeta where meta_key='{$key}' and meta_value='new'");

        wp_send_json(array('new_message' => 0, 'old_count' => $oldcount, 'new_count' => $new_messages, 'unread' => $unread));
    }

    function getMessages(){
        global $wpdb, $current_user;

        if(!is_user_logged_in()) wp_send_json(array('error' => true, 'message' => 'Authorization Failed!'));
        $folder = __::query_var('folder');

        if($folder === 'sent'){
            $sentbox = '__wpdmpm_sentbox_'.$current_user->ID;
            $messages = $wpdb->get_results("select post_id from {$wpdb->prefix}postmeta where meta_key='{$sentbox}'", ARRAY_A);
            $message_ids = array();
            foreach ($messages as $message){
                $message_ids[] = $message['post_id'];
            }
            //wpdmdd("select post_id from {$wpdb->prefix}postmeta where meta_key='{$sentbox}'");
            $messages = $wpdb->get_results("select post_id from {$wpdb->prefix}postmeta where meta_key='__deleted_by_sender' and meta_value < 1", ARRAY_A);
            $message_ids_dels = array();
            foreach ($messages as $message){
                $message_ids_dels[] = $message['post_id'];
            }
            $message_ids = array_intersect($message_ids, $message_ids_dels);
            if(count($message_ids) > 0) {
                $args = array(
                    'post__in' => $message_ids,
                    'post_type' => 'message',
                    'posts_per_page' => -1,
                    'orderby' => 'modified',
                    'order' => 'DESC',
                );
                $messages = new \WP_Query($args);
                $messages = $messages->get_posts();
            } else {
                $messages = array();
            }

        } else if($folder === 'all' && current_user_can(PM_ADMIN_CAP)) {
            $args = array(
                'post_type' => 'message',
                'posts_per_page' => -1,
                'orderby' => 'modified',
                'order' => 'DESC',
            );
            $messages = new \WP_Query($args);
            $messages = $messages->get_posts();


        } else {
            $inbox = '__wpdmpm_inbox_'.$current_user->ID;
            $messages = $wpdb->get_results("select post_id from {$wpdb->prefix}postmeta where meta_key='{$inbox}'", ARRAY_A);
            $message_ids = array();
            foreach ($messages as $message){
                $message_ids[] = $message['post_id'];
            }
            $messages = $wpdb->get_results("select post_id from {$wpdb->prefix}postmeta where meta_key='__deleted_by_receiver' and meta_value < 1", ARRAY_A);
            $message_ids_dels = array();
            foreach ($messages as $message){
                $message_ids_dels[] = $message['post_id'];
            }
            $message_ids = array_intersect($message_ids, $message_ids_dels);
            if(count($message_ids) > 0) {
                $args = array(
                    'post__in' => $message_ids,
                    'post_type' => 'message',
                    'posts_per_page' => -1,
                    'orderby' => 'modified',
                    'order' => 'DESC',
                );
                $messages = new \WP_Query($args);
                $messages = $messages->get_posts();
            } else {
                $messages = array();
            }


        }


        foreach ($messages as &$message){
            $message->post_date = date_i18n(get_option('date_format')." ".get_option('time_format'), strtotime($message->post_modified));
            $message->status = get_post_meta($message->ID, '__msg_status_'.$current_user->ID, true);
            $message->last_reply = (int)get_post_meta($message->ID, '__wpdmpm_last_reply_'.$current_user->ID, true);
            if($message->last_reply > 0) {
                $comauthr = get_comment($message->last_reply);
                $message->last_reply_from = is_object($comauthr) ? $comauthr->comment_author : null;
            }
            else
                $message->last_reply_from = false;
            $message->last_message = (int)get_post_meta($message->ID, '__wpdmpm_last_message_'.$current_user->ID, true);
            if($message->last_message > 0) {
                $comauthr = get_comment($message->last_message);
                $message->last_message_from = is_object($comauthr) ? $comauthr->comment_author : null;
            }
            else
                $message->last_message_from = false;
            $recvr = get_user_by('id', get_post_meta($message->ID, '__receiver', true));
            $message->receiver = is_object($recvr) ? $recvr->data : null;
            unset($message->receiver->user_pass);
            $sendr = get_user_by('id', $message->post_author);
            $message->sender = is_object($sendr) ? $sendr->data : null;
            unset($message->sender->user_pass);


            $message->profile = ['display_name' => '', 'avatar' => ''];

            if($folder === 'sent') {
                if($message->post_author == $current_user->ID)
                    $message->to_name = $message->receiver ? $message->receiver->display_name : '[User# 404]';
                else
                    $message->to_name = $message->sende ? $message->sender->display_name : '[User# 404]';
            } else {
                if($message->post_author == $current_user->ID)
                    $message->from_name = $message->receiver ? $message->receiver->display_name : '[User# 404]';
                else
                    $message->from_name = $message->sender ? $message->sender->display_name : '[User# 404]';
            }

        }
        wp_send_json($messages);


    }

    function getMessage(){
        global $current_user;

        if(!is_user_logged_in()) wp_send_json(array('error' => true, 'message' => 'Authorization Failed!'));
        $folder = __::query_var('folder');

        $id = __::query_var('id');

        $receiver = (int)get_post_meta($id, '__receiver', true);

        $message = get_post($id);
        $message->author_name = get_userdata( $message->post_author )->display_name;
        $message->post_date = date_i18n(get_option('date_format')." ".get_option('time_format'), strtotime($message->post_date));

        //if($receiver->ID !== $current_user->ID && $message->post_author !== $current_user->ID ) wp_send_json(array('error' => true, 'message' => 'Invalid message ID!'));
        $recvr = get_user_by('id', $receiver);
        $message->receiver = is_object($recvr) ? $recvr->data : null;
        if($message->receiver) unset($message->receiver->user_pass);
        $sendr = get_user_by('id', $message->post_author);
        $message->sender = is_object($sendr) ? $sendr->data : null;
        if($message->sender)  unset($message->sender->user_pass);

        $profile = ($current_user->ID === $receiver) ? $sendr->data : $recvr->data;
        $profile->avatar = get_avatar($profile->ID, 512);
        $profile->description = get_user_meta($profile->ID, 'description', true);
        $profile->signup_date = wp_date(get_option('date_format'), strtotime($profile->user_registered));
        $profile->title = get_user_meta($profile->ID, '__wppmsg_title', true);


        $message->profile = $profile; // ['display_name' => $profile->display_name, 'avatar' => get_avatar($profile->ID, 512), 'email' => $profile->user_email, 'description' => $profile->description, 'signup_date' => wp_date(get_option('date_format'), $profile->user_registered)];

        $message->comments = get_comments(array('post_id' => $id, 'order' => 'ASC',  'orderby' => 'comment_date'));

        foreach ($message->comments as &$comment){
            $comment->comment_date = date(get_option('date_format')." ".get_option('time_format'), strtotime($comment->comment_date));
            $cmauthr = get_userdata( $comment->user_id );
            $comment->author_name = is_object($cmauthr) ? $cmauthr->display_name : '[User# 404]';
            $attachments = maybe_unserialize(get_comment_meta($comment->comment_ID, '__wpdmpm_attached_files', true));
            $comment->attachments = array();
            if(is_array($attachments)){
                $attachments = array_unique($attachments);
                foreach ($attachments as $attachment){
                    list($file, ) = explode(".wpdmpm", $attachment);
                    $comment->attachments[] = array('file' => $file, 'download_url' => home_url('/wpdmpm-attachment/'.__Crypt::encrypt(array('file' => $attachment, 'uid' => $current_user->ID))));
                }
            }
        }

        $last_comment = end($message->comments);
        $last_commenter = $last_comment && $last_comment->user_id?$last_comment->user_id:null;
        $message->attachments = array();
        if(!isset($_GET['skip_status'])) //$last_commenter != $current_user->ID && !isset($_GET['skip_status'])
            update_post_meta($id, '__msg_status_'.$current_user->ID, 'read');
        $files = get_post_meta($id, '__wpdmpm_attached_files', true);
        if(is_array($files)) {
            foreach ($files as $attachment) {
                list($file,) = explode(".wpdmpm", $attachment);
                $message->attachments[] = array('file' => $file, 'download_url' => home_url('/wpdmpm-attachment/' . __Crypt::encrypt(array('file' => $attachment, 'uid' => $current_user->ID))));
            }
        }
        wp_send_json($message);


    }

    function sendMessage(){

        global $current_user;

        $sender = $current_user;

        /**
         * Check is user is logged in
         */
        if(!is_user_logged_in()) wp_send_json(array('success' => false, 'message' => 'Only valid users can send a reply!'));

        /**
         * Check required fields
         */
        if(!isset($_POST['subject'], $_POST['message'])) wp_send_json(array('success' => false, 'message' => 'Missing a required data field!'));



        /* User to Admin message ON, Sending to Only Admin */
        /* U2U and U2A both OFF, Default action send to site Admin */
        $__pmtoadmin    = (int)get_option('__pmtoadmin', 0);
        $__pmtouser     = (int)get_option('__pmtouser', 0);
        $__pmadminemail = get_option('__pmadminemail', '');


        if( ( $__pmtoadmin && isset($_POST['admin'])) || (!$__pmtouser && $__pmtoadmin) || !$__pmtoadmin && !$__pmtouser ){
            $admins = get_users(array('role__in' => ('administrator')));
            $user = $admins[0];
            $admin_email = $__pmadminemail !== '' ? $__pmadminemail : $user->user_email;
            $pm_message['email'] = $admin_email;
        }

        /* User to User message ON, not sending to site Admin */
        if(__::query_var('__send_to')){
            $__send_to = isset($_REQUEST['__send_to']) && !empty($_REQUEST['__send_to']) ? __Crypt::decrypt(__::query_var('__send_to')) : false;
            $user = get_user_by('id', $__send_to);
            if(!$user) wp_send_json(array('success' => false, 'message' => 'User email not found!'));
            $pm_message['email'] = $user->user_email;
        }
        else if( $__pmtouser && __::query_var('send_to', 'int') > 0 && !isset($_POST['admin'])) {
            $user = get_user_by( 'ID', __::query_var('send_to', 'int'));
            if(!$user) wp_send_json(array('success' => false, 'message' => 'User email not found!'));
            $pm_message['email'] = $user->user_email;
        }

        if(!isset($user) || !is_object($user) || $user->ID == 0) wp_send_json(array('success' => false, 'message' => 'Missing a required data field!'));

        $mlimit = PrivateMessage()->message->userLimit();

        /* Count messages sent by current user */
        $authorid = get_current_user_id();
        $user_msg_count = 0;
        $user_msg_count = count_user_posts( $authorid , 'message' );

        if($mlimit > $user_msg_count || $mlimit === 0){

            $message = array(
                'post_title'    => $_POST['subject'],
                'post_content'  => $_POST['message'],
                'post_status'   => 'publish',
                'post_author'   => $sender->ID,
                'post_type'     => 'message'
            );

            $msgID = wp_insert_post($message);

            unset($user->user_pass);
            update_post_meta($msgID, '__receiver', $user->ID);
            $message['receiver'][] = $user->data;



            update_post_meta($msgID, '__deleted_by_receiver', 0);
            update_post_meta($msgID, '__deleted_by_sender', 0);

            update_post_meta($msgID, '__msg_status_'.$user->ID, 'new');
            update_post_meta($msgID, '__wpdmpm_sentbox_'.$sender->ID, 1);
            update_post_meta($msgID, '__wpdmpm_inbox_'.$user->ID, 1);
            if(isset($_REQUEST['__attached_files'])){
                update_post_meta($msgID, '__wpdmpm_attached_files', $_REQUEST['__attached_files']);
            }






            $message['sender'] = $sender->data;

            $pmpage = get_option('__pmpage');
            /*
             * Check if download manager active, get the user dashboard page id
             */
            $wpdmudb = function_exists('wpdm_user_dashboard_url') ? get_option('__wpdm_user_dashboard', false) : false;

            /**
             * If PM page is not set or same as download manager user dashboard, use it with udb_page=message
             */
            $url = $pmpage && $wpdmudb !== $pmpage ? get_permalink($pmpage) : wpdm_user_dashboard_url(array('udb_page' => 'messages'));

            $params = array(
                'to_email'      => $pm_message['email'],
                'subject'       => $_POST['subject'],
                'message'       => __::query_var('message', ['validate' => 'escs'])."<a style='margin: 0 !important;display: block;text-align:center;border-radius: 2px' class='button button-green' href='{$url}'>".__('View Message', PM_TEXT_DOMAIN)."</a>"
            );
            $params['name'] = $user->display_name;
            $params['first_name'] = $user->first_name;
            $params['last_name'] = $user->last_name;
            $params['user_login'] = $user->user_login;
            $params['user_id'] = $user->ID;

            __Email::send("private-message", $params);

            wp_send_json(['success' => true, 'message' => __('Message sent successfully!', PM_TEXT_DOMAIN)]);


        } else
            wp_send_json(array('success' => false, 'issue' => 'reached_limit', 'message' => __('You have reached your max sending limit!', PM_TEXT_DOMAIN)));

    }

    function sendMessageAdmin(){

        global $current_user;

        $sender = $current_user;

        /**
         * Check is user is logged in
         */
        if(!is_user_logged_in()) wp_send_json(array('error' => true, 'message' => __('Only valid users can send a reply!', PM_TEXT_DOMAIN)));

        /**
         * Check required fields
         */
        if(!isset($_POST['subject'], $_POST['message'])) wp_send_json(array('error' => true, 'message' => 'Missing a required data field!'));



        /* User to Admin message ON, Sending to Only Admin */
        /* U2U and U2A both OFF, Default action send to site Admin */

        $__pmadminemail = get_option('__pmadminemail', '');
        $__send_msg_to = __::query_var('send_msg_to');

        if($__send_msg_to === 'users') {
            $users = __::query_var('send_to');
            foreach ($users as &$user) {
                $user = get_user_by('id', $user);
            }
        }
        else if($__send_msg_to === 'role') {
            $role = __::query_var('sent_to_role');
            $users = get_users(['role' => $role]);
        }
        else if($__send_msg_to === 'all') {
            $users = get_users();
        }

        $authorid = get_current_user_id();

        /**
         * @var \WP_User $user
         */
        foreach ($users as $user) {

            $message = array(
                'post_title' => $_POST['subject'],
                'post_content' => $_POST['message'],
                'post_status' => 'publish',
                'post_author' => $authorid,
                'post_type' => 'message'
            );

            $msgID = wp_insert_post($message);

            unset($user->user_pass);
            update_post_meta($msgID, '__receiver', $user->ID);
            $message['receiver'][] = $user->data;

            update_post_meta($msgID, '__deleted_by_receiver', 0);
            update_post_meta($msgID, '__deleted_by_sender', 0);

            update_post_meta($msgID, '__msg_status_' . $user->ID, 'new');
            update_post_meta($msgID, '__wpdmpm_sentbox_' . $sender->ID, 1);
            update_post_meta($msgID, '__wpdmpm_inbox_' . $user->ID, 1);
            if (isset($_REQUEST['__attached_files'])) {
                update_post_meta($msgID, '__wpdmpm_attached_files', $_REQUEST['__attached_files']);
            }

            $message['sender'] = $sender->data;

            $pmpage = get_option('__pmpage');
            /*
             * Check if download manager active, get the user dashboard page id
             */
            $wpdmudb = function_exists('wpdm_user_dashboard_url') ? get_option('__wpdm_user_dashboard', false) : false;

            /**
             * If PM page is not set or same as download manager user dashboard, use it with udb_page=message
             */
            $url = $pmpage && $wpdmudb !== $pmpage ? get_permalink($pmpage) : wpdm_user_dashboard_url(array('udb_page' => 'messages'));

            $params = array(
                'to_email' => $user->user_email,
                'subject' => $_POST['subject'],
                'message' => wpautop(__::query_var('message', ['validate' => 'escs']))."<hr/><a style='margin: 0 !important;display: block;text-align:center;border-radius: 2px' class='button button-green' href='{$url}'>View Message</a>"
            );
            $params['name'] = $user->display_name;
            $params['first_name'] = $user->first_name;
            $params['last_name'] = $user->last_name;
            $params['user_login'] = $user->user_login;
            $params['user_id'] = $user->ID;

            __Email::send("private-message", $params);
        }

        wp_send_json(['success' => true, 'total_users' => count($users), 'message' => 'Message sent successfully!']);

    }

    function createReply()
    {
        global $current_user;

        $sender = $current_user;

        if(!$sender->ID) wp_send_json(array('error' => true, 'message' => 'Only valid users can send a reply!'));

        $msgID  = $_POST['message_id'];

        $msg_receiver = get_post_meta($msgID, '__receiver', true);

        $msg = get_post($msgID);

        if((int)$sender->ID !== (int)$msg_receiver && (int)$sender->ID !== (int)$msg->post_author)  wp_send_json(array('error' => true, 'message' => 'You are not authorized!'));

        $reply = strip_tags($_REQUEST['reply'], "<b><strong><p><br><a><img><ul><ol><li>");

        $time = current_time('mysql');

        $data = array(
            'comment_post_ID' => $msgID,
            'comment_author' => $sender->display_name,
            'comment_author_email' => $sender->user_email,
            'comment_content' => wpautop($reply),
            'comment_parent' => 0,
            'user_id' => $sender->ID,
            'comment_author_IP' => __::get_client_ip(),
            'comment_agent' => $_SERVER['HTTP_USER_AGENT'],
            'comment_date' => $time,
            'comment_approved' => 1,
        );

        $replyID = wp_insert_comment($data);

        if(isset($_REQUEST['__attached_files'])){
            update_comment_meta($replyID, '__wpdmpm_attached_files', array_unique($_REQUEST['__attached_files']));
        }

        update_comment_meta($replyID, '__deleted_by_receiver', 0);
        update_comment_meta($replyID, '__deleted_by_sender', 0);

        update_post_meta($msgID, '__wpdmpm_sentbox_'.$sender->ID, 1);
        update_post_meta($msgID, '__wpdmpm_inbox_'.$msg->post_author, 1);

        update_post_meta($msgID, '__wpdmpm_last_reply_'.$sender->ID, $replyID);

        $last_reply_receiver = $current_user->ID == $msg->post_author ? $msg_receiver : $current_user->ID;
        update_post_meta($msgID, '__wpdmpm_last_message_'.$last_reply_receiver, $replyID);

        update_post_meta($msgID, '__wpdmpm_last_reply_receiver', $last_reply_receiver);



        $new_status = $sender->ID == $msg->post_author ? $msg_receiver : $msg->post_author;
        //if($msg_receiver === $sender->ID) {
        update_post_meta($msgID, '__msg_status_'.$new_status, 'new');
        $_post = array(
            'ID' => $msgID,
            'post_excerpt' => time(),
        );
        wp_update_post( $_post );
        //}

        /*
        $sender = $sender->data;
        unset($sender->user_pass);
        $data['sender'] = $sender;
        $receiver = get_user_by('id', get_post_field('post_author', $msgID))->data;
        unset($receiver->user_pass);
        $data['receiver'] = $receiver;
        */
        $mail_receiver = get_userdata($new_status);
        $admins = get_users(array('role__in' => ('administrator')));
        $user = $admins[0];
        $admin_email = PrivateMessage()->settings->general->pm_admin_email !== '' ?  PrivateMessage()->settings->general->pm_admin_email : $user->user_email;
        $to_email = (user_can($mail_receiver, 'manage_options'))?$admin_email:$mail_receiver->user_email;

        $pmpage = get_option('__pmpage');
        /*
         * Check if download manager active, get the user dashboard page id
         */
        $wpdmudb = function_exists('wpdm_user_dashboard_url') ? get_option('__wpdm_user_dashboard', false) : false;

        /**
         * If PM page is not set or same as download manager user dashboard, use it with udb_page=message
         */
        $url = $pmpage && $wpdmudb !== $pmpage ? get_permalink($pmpage) : wpdm_user_dashboard_url(array('udb_page' => 'messages'));

        $params = array(
            'to_email'            => $to_email,
            'subject'       => "[RE# {$replyID}] {$msg->post_title}",
            'message'       => __MailUI::panel("REPLY# {$replyID}", array($data['comment_content']), "<a style='display: block;margin: 0 !important;border-radius: 2px;text-align: center;' class='button button-green' href='{$url}'>View Reply</a>")
        );

        __Email::send("private-message", $params);

        wp_send_json(array('success' => true, 'reply_id' => $replyID));

    }

    function searchPackages(){

        $packs = get_posts(array('post_type' => 'wpdmpro','s' => __::query_var('s', 'txt'), 'posts_per_page' => 10));
        foreach ($packs as $pack){
            $data[] = array('id' => $pack->ID, 'title' => $pack->post_title);
        }
        wp_send_json($data);
        die();

    }

    function categories(){
        $cats = get_terms(array('taxonomy' => 'wpdmcategory',
            'hide_empty' => false));
        foreach ($cats as $cat){
            $data[] = array('value' => $cat->slug, 'id' => $cat->term_id, 'label' => $cat->name);
        }
        wp_send_json($data);
        die();

    }

    function deleteMessage(){
        $message_id = __::query_var('id', 'int');
        $message = get_post($message_id);
        if($message->post_author === get_current_user_id() && $message->post_type === 'message'){
            update_post_meta($message_id, '__deleted_by_sender', 1);
        }
        else if((int)get_post_meta($message_id, '__receiver', true) === get_current_user_id() && $message->post_type === 'message'){
            update_post_meta($message_id, '__deleted_by_receiver', 1);
        }

        wp_send_json(array('success' => true));

    }

    function searchUsers(){
        $user_query = new \WP_User_Query( array( 'search' => "*".__::query_var('keyword', 'txt')."*", 'search_columns' => array( 'user_login', 'user_email' , 'display_name', 'user_nicename' ) )  );
        $result = array();
        if ( ! empty( $user_query->get_results() ) ) {
            foreach ( $user_query->get_results() as $user ) {
                $result[] = array('name' => $user->display_name, 'ID' => $user->ID, 'picture' => get_avatar_url($user->user_email));
            }
        }
        wp_send_json($result);
    }
}
