<?php


namespace PrivateMessage\User;

use PrivateMessage\__\__;
use PrivateMessage\__\__Crypt;
use PrivateMessage\__\__Email;
use PrivateMessage\__\__Session;
use PrivateMessage\__\__Template;
use PrivateMessage\Form\Form;
use PrivateMessage\PrivateMessage;

if(!defined("ABSPATH")) die("Shit happens!");

class Login
{
    private static $instance;

    public static function getInstance()
    {
        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    private function __construct()
    {
        add_action("init", [$this, 'logout']);
        add_action('init', [$this, 'process']);

        add_action("wp_ajax_nopriv_updatePassword", [$this, 'updatePassword']);
        add_action("wp_ajax_nopriv_resetPassword", [$this, 'resetPassword']);

        // Login Form
        add_shortcode('wppmsg_login_form', [$this, 'form']);

    }

    function logout()
    {
        if(__::query_var('wppmsg_logout') !=='' && wp_verify_nonce(__::query_var('wppmsg_logout'), NONCE_KEY))
        {
            wp_logout();
            wp_safe_redirect(__::query_var('redirect_to'));
            die();
        }
    }

    static function formFields($params = [])
    {
        $login_data_fields['__phash'] = ['type' => 'hidden', 'attrs' => ['name' => '__phash', 'id' => '__phash', 'value' => __Crypt::encrypt($params)]];
        $login_data_fields['log'] = array(
            'label' => __("Login ID", "download-manager"),
            'type' => 'text',
            'attrs' => array('name' => 'wppmsg_login[log]', 'id' => 'user_login', 'required' => 'required', 'placeholder' => __('Username or Email', "download-manager")),
        );
        $login_data_fields['password'] = array(
            'label' => __("Password", "download-manager"),
            'type' => 'password',
            'attrs' => array('name' => 'wppmsg_login[pwd]', 'id' => 'password', 'required' => 'required', 'placeholder' => __("Enter Password", "download-manager")),
        );
        if (!isset($params['captcha']) || $params['captcha'] === true) {
            $show_captcha = (int)get_option('__wppmsg_recaptcha_loginform', 0) === 1 && get_option('_wppmsg_recaptcha_secret_key', '') != '';
            $show_captcha = apply_filters("signin_form_captcha", $show_captcha);
            if ($show_captcha) {
                $login_data_fields['__recap'] = array(
                    'type' => 'reCaptcha',
                    'attrs' => array('name' => '__recap', 'id' => '__recap'),
                );
            }
        }
        $login_data_fields = apply_filters("wppmsg_login_form_fields", $login_data_fields);
        $form = new Form($login_data_fields, ['name' => 'wppmsg_login_form', 'id' => 'wppmsg_login_form', 'method' => 'POST', 'action' => '', 'submit_button' => [], 'noForm' => true]);
        return $form->render();

    }

    /**
     * @usage Short-code callback function for login form
     * @return string
     */
    function form($params = array())
    {

        global $current_user;

        if (!isset($params) || !is_array($params)) $params = array();

        if (isset($params) && is_array($params))
            extract($params);
        if (!isset($redirect)) $redirect = get_permalink(get_option('__wppmsg_user_dashboard'));

        if (!isset($regurl)) {
            $regurl = get_option('__wppmsg_register_url');
            if ($regurl > 0)
                $regurl = get_permalink($regurl);
        }
        $log_redirect = $_SERVER['REQUEST_URI'];
        if (isset($params['redirect'])) $log_redirect = esc_url($params['redirect']);
        if (isset($_GET['redirect_to'])) $log_redirect = esc_url($_GET['redirect_to']);

        $up = parse_url($log_redirect);
        if (isset($up['host']) && $up['host'] != $_SERVER['SERVER_NAME']) $log_redirect = $_SERVER['REQUEST_URI'];

        $log_redirect = strip_tags($log_redirect);

        if (!isset($params['logo'])) $params['logo'] = get_site_icon_url();

        $__wppmsg_social_login = get_option('__wppmsg_social_login');
        $__wppmsg_social_login = is_array($__wppmsg_social_login) ? $__wppmsg_social_login : array();

        ob_start();

        if (is_user_logged_in())
            $template = __Template::locate("already-logged-in.php", __DIR__.'/views');
        else {
            if (__::query_var('action') === 'lostpassword')
                $template = __Template::locate('lost-password-form.php', __DIR__.'/views');
            else if (__::query_var('action') === 'rp')
                $template = __Template::locate('reset-password-form.php', __DIR__.'/views');
            else
                $template = __Template::locate('login-form.php', __DIR__.'/views');
        }

        include($template);

        $content = ob_get_clean();
        $content = apply_filters("wppmsg_login_form_html", $content);

        return $content;
    }

    function process()
    {

        global $wp_query, $post, $wpdb;
        if (!isset($_POST['wppmsg_login'])) return;

        $shortcode_params = __Crypt::decrypt(__::query_var('__phash'), true);

        $login_try = (int)__Session::get('login_try');
        $login_try++;
        __Session::set('login_try', $login_try);

        if ($login_try > 30) wp_die("Slow Down!");

        if(!isset($shortcode_params['captcha']) || $shortcode_params['captcha'] ===  true) {
            $active_captcha = (int)get_option('__wppmsg_recaptcha_loginform', 0) === 1 && get_option('_wppmsg_recaptcha_secret_key', '') != '';
            $active_captcha = apply_filters("signin_form_captcha", $active_captcha);
            if ($active_captcha) {
                $ret = __::remote_post('https://www.google.com/recaptcha/api/siteverify', array('secret' => get_option('_wppmsg_recaptcha_secret_key', ''), 'response' => __::query_var('__recap')));
                $ret = json_decode($ret);
                if (!$ret->success) {
                    $login_error = __("Invalid CAPTCHA!", "download-manager");
                    if (__::is_ajax()) {
                        wp_send_json(array('success' => false, 'message' => 'Error: ' . $login_error));
                        die();
                    }
                    __Session::set('login_error', $login_error);
                    wp_safe_redirect(__::query_var('permalink', 'url'));
                    die();
                }
            }
        }

        __Session::clear('login_error');
        $creds = array();
        $creds['user_login'] = isset($_POST['wppmsg_login']['log']) ? $_POST['wppmsg_login']['log'] : '';
        $creds['user_password'] = isset($_POST['wppmsg_login']['pwd']) ? $_POST['wppmsg_login']['pwd'] : '';
        $creds['remember'] = isset($_POST['rememberme']) ? $_POST['rememberme'] : false;
        $user = wp_signon($creds, false);
        if (is_wp_error($user)) {
            $login_error = $user->get_error_message();
            if (__::is_ajax())
                wp_send_json(array('success' => false, 'message' => $login_error));

            __Session::set('login_error', $login_error);
            wp_safe_redirect($_SERVER['HTTP_REFERER']);
            die();
        } else {
            wp_set_auth_cookie($user->ID);
            wp_set_current_user($user->ID);
            __Session::set('login_try', 0);
            //do_action('wp_login', $creds['user_login'], $user);

            if (__::is_ajax())
                wp_send_json(array('success' => true, 'message' => __("Success! Redirecting...", "download-manager")));

            wp_safe_redirect(__::query_var('redirect_to', 'url'));
            die();
        }
    }

    function resetPassword()
    {
        if (__::query_var('__wppmsg_reset_pass')) {

            if (empty($_POST['user_login'])) {
                die('error');
            } elseif (strpos($_POST['user_login'], '@')) {
                $user_data = get_user_by('email', trim(wp_unslash($_POST['user_login'])));
                if (empty($user_data))
                    die('error');
            } else {
                $login = trim($_POST['user_login']);
                $user_data = get_user_by('login', $login);
            }
            if (__Session::get('__reset_time') && time() - __Session::get('__reset_time') < 60) {
                echo "toosoon";
                exit;
            }
            if (!is_object($user_data) || !isset($user_data->user_login)) die('error');
            $user_login = __Crypt::encrypt($user_data->user_login);
            $user_email = $user_data->user_email;
            $key = get_password_reset_key($user_data);


            $reseturl = add_query_arg(array('action' => 'rp', 'key' => $key, 'login' => $user_login), get_permalink(PrivateMessage()->settings->general->pm_login_page_id));

            $params = array('reset_password' => $reseturl, 'to_email' => $user_email, 'user_login' => $user_data->user_login, 'user_email' => $user_data->user_email, 'first_name' => $user_data->first_name, 'last_name' => $user_data->last_name, 'loginurl' => get_permalink(PrivateMessage()->settings->general->pm_page_id));

            __Email::send('password-reset', $params);
            __Session::set('__reset_time', time());
            echo 'ok';
            exit;

        }
    }

    function updatePassword()
    {
        if (__::query_var('__wppmsg_update_pass')) {

            if (wp_verify_nonce(__::query_var('__wppmsg_update_pass'), NONCE_KEY)) {
                $pass = __::query_var('password','html');
                if ($pass == '') die('error');
                $user = __Crypt::decrypt(__::query_var('__up_user'));
                if (is_object($user) && isset($user->ID)) {

                    if(user_can($user->ID, 'manage_options'))
                        wp_send_json(array('success' => false, 'message' => apply_filters('wpdm_update_password_error', __('Password update is disabled for this user!', 'download-manager'))));

                    wp_set_current_user($user->ID, $user->user_login);
                    wp_set_auth_cookie($user->ID);
                    //do_action('wp_login', $user->user_login);
                    wp_set_password($pass, $user->ID);
                    //print_r($user);
                    wp_send_json(array('success' => true, 'message' => ''));
                } else wp_send_json(array('success' => false, 'message' => apply_filters('wpdm_update_password_error', __('Session Expired! Please try again.', 'download-manager'))));
            } else
                wp_send_json(array('success' => false, 'message' => apply_filters('wpdm_update_password_error', __('Session Expired! Please try again.', 'download-manager'))));

        }
    }

}
