<?php
/**
 * Email Handler Class for WordPress Download Manager Pro
 * Since: v4.6.0
 * Author: Shahjada
 */

namespace PrivateMessage\__;

use PrivateMessage\PrivateMessage;

class __Email
{


    function __construct()
    {

    }


    public static function templates()
    {
        $admin_email = get_option('admin_email');
        $sitename = get_option("blogname");
        $templates = array(
            'default' => array('subject' => '[#subject#]',
                'from_name' => PrivateMessage()->settings->email->from_name,
                'from_email' => PrivateMessage()->settings->email->from_email,
                'message' => "[#message#]",
                'template' => "default.html"
            ),
            'user-signup' => array('subject' => '[#subject#]',
                'from_name' => PrivateMessage()->settings->email->from_name,
                'from_email' => PrivateMessage()->settings->email->from_email,
                'message' => PrivateMessage()->settings->email->pm_signup_email_msg_template,
                'template' => PrivateMessage()->settings->email->pm_signup_email_template
            ),
            'user-signup-admin' => array('subject' => '[#subject#]',
                'from_name' => PrivateMessage()->settings->email->from_name,
                'from_email' => PrivateMessage()->settings->email->from_email,
                'message' => PrivateMessage()->settings->email->pm_signup_email_msg_template_admin,
                'template' => PrivateMessage()->settings->email->pm_signup_email_template
            ),
            'password-reset' => array('subject' => sprintf( __( "Request to reset your %s password" , PM_TEXT_DOMAIN ), $sitename ),
                'from_name' => PrivateMessage()->settings->email->from_name,
                'from_email' => PrivateMessage()->settings->email->from_email,
                'message' => PrivateMessage()->settings->email->pm_password_reset_msg_template,
                'template' => PrivateMessage()->settings->email->pm_signup_email_template
            ),
            'private-message' => array('subject' => '[#subject#]',
                'from_name' => PrivateMessage()->settings->email->from_name,
                'from_email' => PrivateMessage()->settings->email->from_email,
                'message' => PrivateMessage()->settings->email->pm_email_msg_template,
                'template' => PrivateMessage()->settings->email->pm_email_template
            ),
            'chat' => array('subject' => '[#subject#]',
                'from_name' => PrivateMessage()->settings->email->from_name,
                'from_email' => PrivateMessage()->settings->email->from_email,
                'message' => PrivateMessage()->settings->email->chat_email_msg_template,
                'template' => PrivateMessage()->settings->email->chat_email_template
            )
        );

        $templates = apply_filters('pm_email_message_templates', $templates);

        return $templates;

    }

    public static function info($id)
    {
        $templates = self::templates();
        return isset($templates[$id]) ? $templates[$id] : $templates['default'];
    }

    public static function tags()
    {
        $tags = array(

            "[#support_email#]" => array('value' => get_option('admin_email'), 'desc' => 'Support Email'),
            "[#img_logo#]" => array('value' => '', 'desc' => 'Site Logo'),
            "[#banner#]" => array('value' => '', 'desc' => 'Banner/Background Image URL'),
            "[#homeurl#]" => array('value' => home_url('/'), 'desc' => 'Home URL of your website'),
            "[#sitename#]" => array(
                'value' => get_option('blogname'),
                'desc' => 'The name/title of your website'
            ),
            "[#site_tagline#]" => array(
                'value' => get_bloginfo('description'),
                'desc' => 'The name/title of your website'
            ),
            "[#loginurl#]" => array('value' => wp_login_url(), 'desc' => 'Login page URL'),
            "[#name#]" => array('value' => '', 'desc' => 'Members First Name'),
            "[#username#]" => array('value' => '', 'desc' => 'Username'),
            "[#password#]" => array('value' => '', 'desc' => 'Members account password'),
            "[#date#]" => array(
                'value' => date_i18n(get_option('date_format'), time()),
                'desc' => 'Current Date'
            ),
            "[#package_name#]" => array('value' => '', 'desc' => 'Package Name'),
            "[#author#]" => array('value' => '', 'desc' => 'Package author profile'),
            "[#package_url#]" => array('value' => '', 'desc' => 'Package URL'),
            "[#edit_url#]" => array('value' => '', 'desc' => 'Package Edit URL')
        );

        $tags["[#client_ip#]"] = ['value' => __::get_client_ip(), 'desc' => 'User IP'];

        if (is_user_logged_in()) {
            global $current_user;
            $tags["[#user_login#]"] = ['value' => $current_user->user_login, 'desc' => 'User login'];
            $tags["[#user_email#]"] = ['value' => $current_user->user_email, 'desc' => 'User email'];
            $tags["[#user_first_name#]"] = ['value' => $current_user->user_firstname, 'desc' => 'User first name'];
            $tags["[#user_last_name#]"] = ['value' => $current_user->user_lastname, 'desc' => 'User last name'];
            $tags["[#user_display_name#]"] = ['value' => $current_user->display_name, 'desc' => 'User display name'];
            $tags["[#user_description#]"] = ['value' => get_user_meta($current_user->ID, 'description', true), 'desc' => 'User display name'];
        }

        return apply_filters("pm_email_template_tags", $tags);
    }


    public static function prepare($id, $params)
    {

        $template = self::info($id);

        $template_data = self::loadTemplateFile($template['template']);

        $logo = \PrivateMessage()->settings->email->logo;
        $banner = \PrivateMessage()->settings->email->banner;
        $params['img_logo'] = $logo != '' ? "<img style='max-width: 70%' src='{$logo}' alt='" . esc_attr(get_option('blogname')) . "' />" : "";
        $params['banner'] = isset($params['banner']) && $params['banner'] != '' ? esc_url($params['banner']) : "";
        $params['banner_img'] = isset($params['banner']) && $params['banner'] != '' ? "<img style='max-width: 100%;' src='{$banner}' alt='Banner Image' />" : "";
        $params['footer_text'] = \PrivateMessage()->settings->email->footer_text;

        $template['message'] = str_replace("[#message#]", stripslashes(wpautop($template['message'])), $template_data);

        $tags = self::tags();
        $vars = array();
        foreach ($params as $key => $val) {
            $vars["[#{$key}#]"] = stripslashes($val);
        }

        foreach ($tags as $key => $info) {
            if (!isset($vars[$key])) {
                $vars[$key] = $info['value'];
            }
        }
        if(isset($params['subject']) && $params['subject'] !== '')
            $template['subject'] = $params['subject'];
        $template['subject'] = str_replace(array_keys($vars), array_values($vars), $template['subject']);
        $template['to_email'] = $params['to_email'];
        $template['cc'] = __::valueof($params, 'cc');
        $template['bcc'] = __::valueof($params, 'bcc');
        $template['message'] = str_replace(array_keys($vars), array_values($vars), $template['message']);
        $template['message'] = self::compile($template['message']);

        return $template;
    }

    public static function send($id, $params)
    {

        $template = self::prepare($id, $params);

        $headers[] = "From: " . $template['from_name'] . " <" . $template['from_email'] . ">";
        $headers[] = "Content-type: text/html";

        $to = $params['to_email'];

        if (isset($params['cc'])) {
            $headers[] = "CC: {$params['cc']}";
            unset($params['cc']);
        }
        if (isset($params['bcc'])) {
            $headers[] = "Bcc: {$params['bcc']}";
            unset($params['bcc']);
        }
        $message = $template['message'];

        return wp_mail($to, html_entity_decode($template['subject']), $message, $headers, __::valueof($params, 'attachments'));
    }


    public function preview()
    {
        global $current_user;
        if (!isset($_REQUEST['action']) || $_REQUEST['action'] != 'email_template_preview') {
            return;
        }
        if (!current_user_can(PM_ADMIN_CAP)) {
            die('Error');
        }
        $id = $_REQUEST['id'];
        $email = self::info($id);
        $params = array(
            "name" => $current_user->display_name,
            "username" => $current_user->user_login,
            "password" => "**************",
            "package_name" => __("Sample Package Name", "download-manager"),
            "author" => $current_user->display_name,
            "package_url" => "#",
            "edit_url" => "#"
        );

        if (isset($_REQUEST['etmpl'])) {
            $params['template_file'] = $_REQUEST['etmpl'];
        }
        $template = self::prepare($id, $params);
        echo $template['message'];
        die();

    }

    static public function fetch($template, $message)
    {
        global $current_user;
        if (!current_user_can(PM_ADMIN_CAP)) {
            die('Error');
        }

        $params['template_file'] = $template;

        $template = self::prepare('default', $params);
        return $template['message'];
    }

    static function compile($template, $rule = "/\[\#(.*)\#\]/")
    {
        $compiled = preg_replace_callback($rule, [new self, '_var'], $template);
        return $compiled;
    }

    static function _var($matched)
    {

        if (substr_count($matched[1], "acf_user_meta_")) {
            $meta_name = str_replace("acf_user_meta_", "", $matched[1]);
            $data = maybe_unserialize(get_user_meta(get_current_user_id(), 'wpdm_cregf', true));
            $value = __::valueof($data, $meta_name);
            if (is_array($value)) $value = implode(", ", $value);
            return $value;
        }
        if (substr_count($matched[1], "user_meta_")) {
            $meta_name = str_replace("user_meta_", "", $matched[1]);
            if (substr_count($meta_name, '/')) {
                $meta_name = explode("/", $meta_name);
                $meta_value = get_user_meta(get_current_user_id(), $meta_name[0], true);
                array_shift($meta_name);
                $meta_value = __::valueof($meta_value, implode("/", $meta_name));
                return $meta_value;
            }
            return get_user_meta(get_current_user_id(), $meta_name, true);
        }
        if (substr_count($matched[1], "SERVER_")) {
            $meta_name = str_replace("SERVER_", "", $matched[1]);
            if (substr_count($meta_name, '/')) {
                $meta_name = explode("/", $meta_name);
                $meta_value = get_user_meta(get_current_user_id(), $meta_name[0], true);
                array_shift($meta_name);
                $meta_value = __::valueof($meta_value, implode("/", $meta_name));
                return $meta_value;
            }
            return get_user_meta(get_current_user_id(), $meta_name, true);
        }
        if (substr_count($matched[1], "REQUEST_")) {
            $meta_name = str_replace("REQUEST_", "", $matched[1]);
            $meta_value = __::valueof($_REQUEST, $meta_name);
            if (is_array($meta_value)) $meta_value = implode(", ", $meta_value);
            return $meta_value;
        }
        return $matched[1];
    }

    static function loadTemplateFile($file)
    {
        $email_template_dirs = array(
            PM_BASE_DIR . 'src/__/views/email-templates',
            get_template_directory() . '/private-message/email-templates',
            get_stylesheet_directory() . '/private-message/email-templates',

        );

        $__templates = array();
        foreach ($email_template_dirs as $email_template_dir) {
            if (file_exists($email_template_dir)) {
                $_templates = scandir($email_template_dir);
                foreach ($_templates as $template) {
                    if (strstr($template, '.html')) {
                        $__templates[$template] = $email_template_dir . '/' . $template;
                    }
                }
            }
        }

        return file_get_contents($__templates[$file]);

    }


}

