<?php
/*
Plugin Name: WPDM - Amazon s3
Plugin URI: https://www.wpdownloadmanager.com/download/amazon-s3-storage-add-on/
Description: Amazon s3 add-on for WordPress Download Manager
Author: WordPress Download Manager
Version: 3.0.3
Author URI: https://www.wpdownloadmanager.com/
*/

if (!defined('WPDM_CLOUD_STORAGE'))
    define('WPDM_CLOUD_STORAGE', 1);

use Aws\S3\S3MultiRegionClient;
use Aws\S3\Exception\S3Exception;
use Aws\Credentials\Credentials;
use Aws\S3\MultipartUploader;

if (defined('WPDM_VERSION')) {

    require_once dirname(__FILE__) . '/aws-sdk/vendor/autoload.php';

    define('WPDM_AWS_URL', plugins_url('wpdm-amazon-s3/'));

    class awsExplorer
    {
        protected $s3Client;

        function __construct()
        {


            add_action('wp_ajax_awsDashboard', array($this, 'awsDashboard'));
            add_action('wp_ajax_awsBuckets', array($this, 'awsBuckets'));
            add_action('wp_ajax_exploreBucket', array($this, 'exploreBucket'));
            add_action('wp_ajax_exploreObject', array($this, 'exploreObject'));
            add_action('wp_ajax_createObject', array($this, 'createObject'));
            add_action('wp_ajax_awsAccess', array($this, 'awsAccess'));
            add_action('wp_ajax_deleteFile', array($this, 'deleteFile'));
            add_action('before_delete_post', array($this, 'deletePackageFiles'), 10, 2);
            $this->s3Client();
        }

        function s3Client()
        {
            $key = WPDM_AmazonS3::getS3Cred();
            if (!$key || !is_array($key)) return false;
            $credentials = new Credentials($key['access'], $key['secret']);
            $params = array(
                'credentials' => $credentials
            );
            if (isset($_REQUEST['region']) && $_REQUEST['region'] != '') {
                $params['version'] = 'latest';
                $params['region'] = wpdm_query_var('region');
            } else {
                $params['version'] = 'latest';
                $params['region'] = 'us-west-2';
            }
            $this->s3Client = new S3MultiRegionClient($params);  //S3Client::factory($params);
            //$buckets = $this->s3Client->listBuckets();
            //wpdmdd($buckets);
            return $this->s3Client;
        }

        function awsAccess()
        {
            if (!current_user_can(WPDM_ADMIN_CAP)) die();
            $access = WPDM_AmazonS3::getS3Cred(); //['access'=> get_option('_wpdm_aws_access_key'), 'secret' => get_option('_wpdm_aws_secret_key')];
            header('Content-type: application/json');
            echo json_encode($access);
            die();
        }

        function awsDashboard()
        {
            die('<div class="alert alert-info">Select Bucket From Left</div>');
        }

        function awsBuckets()
        {
            $buckets = $this->s3Client->listBuckets();
            $buckets = $buckets['Buckets'];
            $_buckets = [];
            foreach ($buckets as $bucket) {
                /*$region = $this->s3Client->getBucketLocation([
                    'Bucket' => $bucket['Name'],
                ]);*/
                //$region['Location'] = '';

                $_buckets[] = $bucket['Name'];

                if (wpdm_query_var('datatype') !== 'json') {
                    ?>
                    <li class="list-group-item">
                    <a href="#" class="explore-bucket" data-region="<?php echo $region['Location']; ?>"
                       data-bucket="<?php echo $bucket['Name']; ?>"><i
                                class="fab fa-dropbox text-primary"></i> <?php echo $bucket['Name']; ?></a>
                    </li><?php
                }
            }
            if (wpdm_query_var('datatype') === 'json')
                wp_send_json(['success' => true, 'buckets' => $_buckets]);
            die();
        }

        function awsBreadcrumb($bucket, $prefix)
        {
            $prefix = explode('/', $prefix);
            $links = array();
            $tpx = '';
//            $region = $this->s3Client->getBucketLocation([
//                'Bucket' => $bucket['Name'],
//            ]);
            $links[] = "<a class='explore-bucket' data-region='' data-bucket=\"$bucket\" data-prefix=\"\" href=\"#\" ><i class='fa fa-map-marker-alt'></i> " . $bucket . "</a>";
            foreach ($prefix as $px) {
                if ($tpx != '')
                    $tpx .= "/$px";
                else $tpx = $px;
                $links[] = "<a class='explore-object' data-region='' data-bucket=\"$bucket\" data-prefix=\"$tpx\" href=\"#\" >" . $px . "</a>";
            }
            return '<div id="breadcrumb" class="breadcrumb">' . implode(' &nbsp;<i class="fa fa-angle-right"></i>&nbsp; ', $links) . '</div>';
        }

        function exploreBucket()
        {
            global $current_user;

            $selected_buckets = get_option('__wpdm_s3_bucket');
            $selected_bucket = wpdm_valueof($selected_buckets, $current_user->roles[0]);

            $wpdm_s3_readonly = get_option('__wpdm_s3_readonly');
            $wpdm_s3_readonly = wpdm_valueof($wpdm_s3_readonly, $current_user->roles[0], ['validate' => 'int']);

            $bucket = wpdm_query_var('bucket', ['default' => $selected_bucket]);

            if (!wpdm_query_var('prefix')) {
                $result = $this->s3Client->putBucketCors([
                    'Bucket' => $bucket, // REQUIRED
                    'CORSConfiguration' => [ // REQUIRED
                        'CORSRules' => [ // REQUIRED
                            [
                                'AllowedHeaders' => ['*'],
                                'AllowedMethods' => ['POST', 'GET', 'PUT', 'HEAD', 'DELETE'], // REQUIRED
                                'AllowedOrigins' => ['*'], // REQUIRED
                                'ExposeHeaders' => ['ETag'],
                                'MaxAgeSeconds' => 3000
                            ],
                        ],
                    ]
                ]);
            }

            $region = $this->s3Client->getBucketLocation([
                'Bucket' => $bucket,
            ]);
            //wpdmprecho($region);
            $lifetime = 86400;
            try {

                $params = array('Bucket' => $bucket, 'Delimiter' => '/');
                if ($region['Location'] != '') {
                    $params['signature'] = 'v4';
                    $params['Location'] = $region['Location'];
                }
                $files = $this->s3Client->ListObjects($params); //getIterator('ListObjects', $params);

                if (wpdm_query_var('datatype') === 'json') {
                    $folders = [];
                    foreach ($files['CommonPrefixes'] as $item) {
                        //if (substr_count($item['Key'], '/') === 1 && preg_match("/\/$/", $item['Key']))
                        $folders[] = trim($item['Prefix'], '/');
                    }
                    wp_send_json(['success' => true, 'folders' => $folders]);
                }


            } catch (S3Exception $e) {
                if (wpdm_query_var('datatype') === 'json')
                    wp_send_json(['success' => false, 'error' => $e->getMessage()]);
                else
                    die("<div class='panel panel-danger'><div class='panel-heading'><i class='fa fa-warning'> </i> Error!</div><div class='panel-body text-danger'>" . $e->getMessage() . "</div></div>");
            }

            echo '<ul class="list-group list-group-flush">';
            $list = array();
            foreach ($files['CommonPrefixes'] as $item) {
                $folder = trim($item['Prefix'], '/');
                $list[$folder] = ['type' => 'd', 'Key' => $item['Prefix']];
            }
            foreach ($files['Contents'] as $file_info) {

                /*if (strstr($file_info['Key'], '/')) {
                    $folder = $file_info['Key'];
                    $folder = explode("/", $folder);
                    $prefix = $folder[0];
                    $file_info['type'] = 'd';
                    //$prefix = rtrim(str_replace(basename($folder), "", $folder), '/');
                    if ($prefix != '')
                        $list[$prefix] = $file_info;

                } else {
                    $file_info['type'] = 'f';
                    $list[$file_info['Key']] = $file_info;
                }*/

                $file_info['type'] = 'f';
                $list[$file_info['Key']] = $file_info;

            }


            foreach ($list as $prefix => $file_info) {

                $ext = preg_replace('/^.*\./', '', $file_info['Key']);
                $ext = 'file ext_' . $ext;
                if (strstr($file_info['Key'], '/')) $ext = 'directory collapsed';
                if ($file_info['Owner']['DisplayName'] != 's3-log-service') {
                    $furl = "https://{$bucket}.s3.amazonaws.com/" . $file_info['Key']; //'$bucket', '$prefix'
                    $durl = $this->GetAuthenticatedURL($bucket, $file_info['Key'], $lifetime);
                    if ($file_info['type'] == 'f') {
                        $attach_link = wpdm_query_var('__s3picker', 'int') === 1 ? "<a class='text-primary attach-file' href='#' data-file=\"{$furl}\"><i class='fa fa-link'></i> Attach</a>" : "";
                        $delete_link = $wpdm_s3_readonly ? "" : "<a href='#' data-bucket=\"$bucket\" data-prefix=\"$prefix\" class='delete-file text-danger'><i class='fas fa-trash'></i> Delete</a>";
                        $download_link = "<a href=\"{$durl}\" class='text-success'><i class='far fa-arrow-alt-circle-down'></i> Download</a>";
                        echo "<li class='list-group-item'><div class='pull-right aactions'>{$download_link}{$attach_link}{$delete_link}</div><i class='fa fa-file text-success'></i> &nbsp;" . $prefix . "</li>";
                    } else {
                        $delete_link = $wpdm_s3_readonly ? "" : "<a href='#' data-bucket=\"$bucket\" data-prefix=\"$prefix\" class='delete-file text-danger'><i class='fas fa-trash'></i> Delete</a>";
                        echo "<li class='list-group-item'><div class='pull-right aactions'>{$delete_link}</div><a href=\"#\" class='explore-object' data-region='{$region['Location']}'  data-bucket=\"$bucket\" data-prefix=\"$prefix\"><i class='fa fa-folder text-info'></i> &nbsp;" . $prefix . "</a></li>";
                    }
                }


            }

            echo '</ul>';
            die();
        }

        function exploreObject()
        {
            global $current_user;

            $selected_buckets = get_option('__wpdm_s3_bucket');
            $selected_bucket = wpdm_valueof($selected_buckets, $current_user->roles[0]);

            $bucket = wpdm_query_var('bucket', ['default' => $selected_bucket]);
            $prefix = !wpdm_query_var('prefix') ? '' : wpdm_query_var('prefix') . '/';
            $lifetime = 3600;

            //$files = $this->s3Client->listObjects(array('Bucket'=>$bucket, 'Prefix' => $prefix));
            $params = ['Bucket' => $bucket, 'Prefix' => $prefix];
            $files = $this->s3Client->getIterator('ListObjects', $params);
            $region = wpdm_query_var('region');
            if($prefix)
                echo $this->awsBreadcrumb($bucket, $prefix);
            echo '<ul class="list-group list-group-flush">';
            $list = array();

            foreach ($files as $file_info) {
                $folder = str_replace($prefix, '', $file_info['Key']);
                if (strstr($folder, '/')) {
                    $folder = explode("/", $folder);
                    $nprefix = $folder[0];
                    $file_info['type'] = 'd';
                    //$prefix = rtrim(str_replace(basename($folder), "", $folder), '/');
                    if (!array_key_exists($nprefix, $file_info) && $nprefix != '')
                        $list[$nprefix] = $file_info;

                } else {
                    $file_info['type'] = 'f';
                    if ($folder != '')
                        $list[$file_info['Key']] = $file_info;

                }

            }


            foreach ($list as $nprefix => $file_info) {

                $ext = preg_replace('/^.*\./', '', $file_info['Key']);
                $ext = 'file ext_' . $ext;
                if (strstr($file_info['Key'], '/')) $ext = 'directory collapsed';
                if ($file_info['Owner']['DisplayName'] != 's3-log-service') {
                    $furl = "https://{$bucket}.s3.amazonaws.com/" . $file_info['Key'];
                    if ($file_info['type'] == 'f') {
                        $durl = $this->GetAuthenticatedURL($bucket, $file_info['Key'], $lifetime);
                        $attach_link = wpdm_query_var('__s3picker', 'int') === 1 ? "<a class='text-primary attach-file' href='#' data-file=\"{$furl}\"><i class='fa fa-link'></i> Attach</a>" : "";
                        echo "<li class='list-group-item'><div class='pull-right aactions'><a href=\"{$durl}\" class='text-success'><i class='fa fa-download'></i> Download</a>{$attach_link}<a href='' data-bucket=\"$bucket\" data-prefix=\"{$nprefix}\" class='delete-file text-danger'><i class='fas fa-trash'></i> Delete</a></div><a id=\"" . uniqid() . "\" href=\"#\" rel=\"" . $furl . "\"><i class='fa fa-file text-success'></i> &nbsp;" . basename($furl) . "</a></li>";
                    } else
                        echo "<li class='list-group-item'><div class='pull-right aactions'><a href='' data-bucket=\"$bucket\" data-prefix=\"{$nprefix}\" class='delete-file text-danger'><i class='fas fa-trash'></i> Delete</a></div><a href=\"#\" class='explore-object' data-region='{$region}' data-bucket=\"$bucket\" data-prefix=\"{$prefix}{$nprefix}\"><i class='fa fa-folder text-info'></i> &nbsp;" . $nprefix . "</a></li>";
                }


            }

            echo '</ul>';
            die();
        }

        function createObject()
        {
            $bucket = $_POST['bucket'];
            $folder = $_POST['folder'];
            $prefix = isset($_POST['prefix']) && !empty($_POST['prefix']) ? $_POST['prefix'] . '/' : '';

            try {
                $this->s3Client->putObject(array(
                    'Bucket' => $bucket,
                    'Key' => "{$prefix}{$folder}/",
                    'Body' => ""
                ));
                die('Done!');
            } catch (S3Exception $e) {
                die($e->getMessage());
            }
        }

    }

    class WPDM_AmazonS3 extends awsExplorer
    {

        protected $accesskey;
        protected $privatekey;

        function __construct()
        {

            parent::__construct();

            register_activation_hook(__FILE__, array($this, 'dbUpdate'));
            add_action('upgrader_process_complete', array($this, 'pluginUpdate'), 10, 2);

            add_action('manage_wpdmpro_posts_columns', array($this, 'columnsTH'));
            add_action('manage_wpdmpro_posts_custom_column', array($this, 'columnsTD'), 10, 2);

            add_action("plugins_loaded", array($this, 'autoSync'), -1);

            add_action('wpdm_after_upload_file_frontend', array($this, 'autoUpload'), 99);
            add_action('wpdm_after_upload_file', array($this, 'autoUpload'), 99);

            //add_filter('wpdm_meta_box', array($this, 'Metabox'));
            add_filter('wpdm_cloud_storage_settings', array($this, 'settings'));
            add_action('wpdm_onstart_download', array($this, 'download'), 9999999);
            add_action('admin_menu', array($this, 'Menu'));
            add_action('wp_ajax_CreateBucket', array($this, 'CreateBucket'));
            //add_action('wp_ajax_s3PackageOptions', array($this, 's3PackageOptions'));
            add_action('wpdm_attach_file_metabox', array($this, 's3AttachFile'));
            add_action('wpdm_package_settings_tr', array($this, 'packageSettings'));

            add_action('admin_init', array($this, 'filePicker'));
            add_action('admin_head', array($this, 'adminHead'));

            if (!isset($_REQUEST['wpdmaws'])) return;

            $key = self::getS3Cred();
            if ($key && isset($key['access'])) {
                $this->accesskey = $key['access']; //get_option('_wpdm_aws_access_key');
                $this->privatekey = $key['secret']; //get_option('_wpdm_aws_secret_key');
            }

            add_action('wp_ajax_ListBuckets', array($this, 'ListBuckets'));
            add_action('wp_ajax_GetBucket', array($this, 'GetBucket'));
            add_action('wp_ajax_wpdm_aws_upload', array($this, 'upload'));


        }

        function dbUpdate()
        {
            global $wpdb;
            $sqls = [];
            $sqls[] = "CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}ahm_amazons3_sync` (
                          `ID` int(11) NOT NULL AUTO_INCREMENT,
                          `PID` int(11) NOT NULL,
                          `status` tinyint(1) NOT NULL,
                          `time` int(11) NOT NULL,
                          PRIMARY KEY (`ID`),
                          UNIQUE KEY `PID` (`PID`)
                       )";
            foreach ($sqls as $sql) {
                $wpdb->query($sql);
            }
        }

        function pluginUpdate($upgrader_object, $options)
        {
            $current_plugin_path_name = plugin_basename(__FILE__);

            if ($options['action'] == 'update' && $options['type'] == 'plugin') {
                foreach ($options['plugins'] as $each_plugin) {
                    if ($each_plugin == $current_plugin_path_name) {
                        $this->dbUpdate();
                    }
                }
            }
        }

        function s3PackageOptions(){
            $ID = wpdm_query_var('pid', 'int');
            include __DIR__.'/tpls/aws-package-settings.php';
            die();
        }

        function packageSettings(){
            $autoe = (int)get_post_meta(wpdm_query_var('post'),'__wpdm_auto_sync', true);
            ?>
            <tr id="__wpdm_auto_sync">
                <td><?php _e( "S3 Auto Sync:", "wpdm-amazon-s3" ) ?></td>
                <td>
                    <label><input type="radio" name="file[auto_sync]" value="0" <?php checked(0, $autoe) ?> > <?php _e( "Use Global", "wpdm-amazon-s3" ) ?> &nbsp; </label>
                    <!-- label style="margin-right: 10px"><input type="radio" name="__s3wpdm_auto_sync" value="1" <?php checked(1, $autoe) ?>> <?php _e( "Enable", "wpdm-amazon-s3" ) ?></label -->
                    <label style="margin-right: 20px"><input type="radio" name="file[auto_sync]" value="-1" <?php checked(-1, $autoe) ?>> <?php _e( "Disable", "wpdm-amazon-s3" ) ?></label>

                </td>
            </tr>
            <?php
        }

        function columnsTH($defaults)
        {
            //if(get_post_type() !== 'wpdmpro') return $defaults;
            if(!(int)get_option('__wpdm_s3_autosync', 0)) return $defaults;
            $defaults['wpdms3'] = "<div class='w3eden'><span class='ttip' title='Amazon S3 sync status' style='font-size: 0.8em'><i class='fab fa-amazon'></i></span></div>";
            return $defaults;
        }


        function columnsTD($column_name, $post_ID)
        {
            //if(get_post_type()!='wpdmpro') return;
            if ($column_name === 'wpdms3') {
                $title = get_the_title($post_ID);
                $sync_time = (int)get_post_meta($post_ID, '__s3wpdm_sync_status', true);
                $color = $sync_time > 0 ? 'var(--color-success)' : 'rgba(0,0,0,0.2);';
                $stitle = $sync_time > 0 ? 'Synced on ' . wp_date(get_option('date_format') . " H:i:s A", $sync_time) : 'Not Synced Yet';
                echo "<div class='w3eden'><i title='{$stitle}' class='ttip fa fa-check-double' style='color: {$color}'></i></div>";
                //<a href='#' data-title='{$title}' data-pid='{$post_ID}' class='pconfig-s3'><i class='fa fa-cog' style='color: var(--color-info)'></i></a>
            }

        }

        function autoSync()
        {
            if (wpdm_query_var('s3sync', 'int') === 1) {
                if((int)get_option('__wpdm_s3_autosync', 0) === 0) wpdmdd('Auto-Sync is not active!');
                global $wpdb;
                $sync_queue_id = get_option("__wpdm_sync_queue_id", 0);
                $ID = $wpdb->get_var("select ID from {$wpdb->prefix}posts where post_type='wpdmpro' and post_status='publish' and ID > {$sync_queue_id} limit 0, 1");
                $this->syncPackage($ID);
                update_post_meta($ID, '__s3wpdm_sync_status', time());
                update_option("__wpdm_sync_queue_id", $ID);
                wpdmdd("Package #{$ID} is synced!");
            }
        }

        function syncPackage($ID)
        {
            $files = WPDM()->package->getFiles($ID, true);
            $bucket = get_option('__wpdm_s3_autosync_bucket', '');
            $folder = get_option('__wpdm_s3_autosync_folder', '');
            if ($bucket === '') wpdmdd("Please select a bucket to auto-sync from wpdm cloud storage settings");
            $key = self::getS3Cred();
            if ($key && isset($key['access'])) {
                $this->accesskey = $key['access']; //get_option('_wpdm_aws_access_key');
                $this->privatekey = $key['secret']; //get_option('_wpdm_aws_secret_key');
                foreach ($files as &$file) {
                    $file = WPDM()->fileSystem->absPath($file);
                    $filename = str_replace(ABSPATH, '', $file);
                    if (file_exists($file)) {
                        $ret = $this->s3Client->putObject(array('Bucket' => trim($bucket, "/"), 'Key' => $folder . '/' . $filename, 'SourceFile' => $file));
                        $file = $ret['ObjectURL'];
                    }

                }
            }
            update_post_meta($ID, '__wpdm_files', $files);
        }


        function Menu()
        {
            add_submenu_page('edit.php?post_type=wpdmpro', __('Amazon S3 &lsaquo; Download Manager', "wpdmpro"), __('Amazon S3', "wpdmpro"), WPDM_MENU_ACCESS_CAP, 'wpdm-aws', array($this, 'awsManager'));
        }

        function awsManager()
        {
            include "tpls/aws-manager.php";
        }

        function filePicker()
        {
            if (isset($_REQUEST['s3browser'])) {
                include "tpls/aws-file-picker.php";
                die();
            }
        }

        function autoUpload($file){
            $ddz = (int)get_option('__wpdm_s3_direct_dropzone');
            $pdir = (int)get_option('__wpdm_s3_packagedir');
            if($ddz)
            {
                $filename = wpdm_basename($file);
                $bucket = get_option('__wpdm_s3_ddz_bucket');
                $folder = get_option('__wpdm_s3_ddz_folder', '');
                if ($pdir)
                    $folder .= "/" . wpdm_query_var('package_id');
                $url = "https://{$bucket}.s3.amazonaws.com/" . ($folder ? $folder . '/' : "") . $filename;
                $this->s3Client->putObject(array('Bucket' => trim($bucket, "/"), 'Key' => $folder . '/' . $filename, 'SourceFile' => $file));

                @unlink($file);

                die("|||$url|||");
            }
        }

        function s3AttachFile()
        {
            $ddz = (int)get_option('__wpdm_s3_direct_dropzone');
            ?>
            <div class="w3eden">
                <?php if($ddz) { ?>

                    <div class="alert alert-info" style="margin: 10px 0">
                        <em class="note"><?php _e( "All uploaded files will be stored in the specified bucket/folder of Amazon S3", "wpdm-amazon-s3" ) ?></em>
                        <pre><?php echo get_option("__wpdm_s3_ddz_bucket"); $f = get_option("__wpdm_s3_ddz_folder"); if($f) echo "/{$f}"; ?></pre>
                    </div>

                <?php } ?>

                <button type="button" style="margin-top: 10px" class="btn btn-primary btn-block"
                        id="s3browser"><?php _e("Select From Amazon S3", "wpdm-amazon-s3") ?></button>
            </div>


            <script>



                jQuery(function ($) {

                    $('#s3browser').on('click', function () {
                        wpdm_iframe_modal("<?php echo admin_url("/index.php?s3browser=1"); ?>", true);
                    });

                });



                function amazons3_attach_file(file) {

                    var ext = file.split('.');
                    ext = ext[ext.length - 1];
                    var icon = "<?php echo WPDM_BASE_URL; ?>file-type-icons/" + ext + ".png";
                    var d = new Date();
                    var ID = d.getTime();

                    var _file = {};
                    _file.filetitle = file;
                    _file.filepath = file;
                    _file.fileindex = ID;
                    _file.preview = icon;
                    <?php if(version_compare(WPDM_VERSION, '4.0.0', '>')){  ?>
                    wpdm_attach_file(_file);
                    <?php } else { ?>
                    jQuery('#wpdmfile').val(file);
                    jQuery('#cfl').html('<div><strong>' + file + '</strong>').slideDown();
                    <?php } ?>
                }
            </script>

            <?php
        }

        function ListBuckets()
        {

            if (!isset($_GET['wpdmaws'])) return;

            if (isset($_POST['dir']) && $_POST['dir'] != 'awsbuckets') {
                $this->GetBucket();
                die();
            }

            $buckets = $this->s3Client->listBuckets();

            $buckets = $buckets['Buckets'];

            echo "<ul class=\"jqueryFileTree\" style=\"display: none;\">";
            // All dirs
            if (!\WPDM\__\Session::get('current_bucket')) {
                \WPDM\__\Session::set('current_bucket', $buckets[0]['Name']);
            }
            foreach ($buckets as $bucket) {
                $bucket = $bucket['Name'];
                $cx = \WPDM\__\Session::get('current_bucket') == $bucket ? 'expanded' : 'collapsed';
                if (\WPDM\__\Session::get('current_bucket') != '' && \WPDM\__\Session::get('current_bucket') == $bucket && 1 == 2) {
                    $stree = "<ul class=\"jqueryFileTree\" style=\"display: none;\">";
                    $files = $this->GetBucket($bucket);
                    if (is_array($files)) {
                        foreach ($files as $name => $file_info) {
                            $ext = preg_replace('/^.*\./', '', $name);
                            $furl = "https://{$bucket}.s3.amazonaws.com/" . $name;
                            $stree .= "<li class=\"file ext_$ext\"><a id=\"" . uniqid() . "\" href=\"#\" rel=\"" . $furl . "\">" . $file_info['name'] . "</a></li>";

                        }
                    }

                    $stree .= "</ul>";
                } else
                    $stree = "";
                echo "<li class=\"directory {$cx}\"><a id=\"" . uniqid() . "\" href=\"#\" rel=\"" . $bucket . "/\">" . $bucket . "</a>{$stree}</li>";

            }
            echo "</ul>";

            die();
        }

        function GetBucket()
        {

            if (!isset($_GET['wpdmaws'])) return;
            //$_POST['dir'] = rtrim($_POST['dir'],'/');
            $dir = explode('|', urldecode($_POST['dir']));
            $bucket = $dir[0];
            $prefix = isset($dir[1]) ? $dir[1] : '';
            \WPDM\__\Session::set('current_bucket', $bucket);

            echo "<ul class=\"jqueryFileTree\" style=\"display: none;\">";
            // All dirs
            //dd(array('Bucket'=>$bucket, 'Prefix' => $prefix));
            //$files = $this->s3Client->listObjects(array('Bucket'=>rtrim($bucket,'/'), 'Prefix' => $prefix));

            $files = $this->s3Client->getIterator('ListObjects', array('Bucket' => rtrim($bucket, '/'), 'Prefix' => $prefix));

            $list = array();

            //$prefix = rtrim($prefix, '/');
            foreach ($files as $file_info) {
                $folder = str_replace($prefix, '', $file_info['Key']);
                if (strstr($folder, '/')) {
                    $folder = explode("/", $folder);
                    $nprefix = $folder[0];
                    $file_info['type'] = 'd';
                    //$prefix = rtrim(str_replace(basename($folder), "", $folder), '/');
                    if (!array_key_exists($nprefix, $file_info) && $nprefix != '')
                        $list[$nprefix] = $file_info;

                } else {
                    $file_info['type'] = 'f';
                    if ($folder != '')
                        $list[$folder] = $file_info;

                }

            }

            foreach ($list as $nprefix => $file_info) {

                $ext = preg_replace('/^.*\./', '', $file_info['Key']);
                $ext = 'file ext_' . $ext;
                //if(strstr($file_info['Key'], '/')) $ext = 'directory collapsed';
                if ($file_info['Owner']['DisplayName'] != 's3-log-service') {
                    $bucketName = rtrim($bucket, '/');
                    $furl = "https://{$bucketName}.s3.amazonaws.com/" . $file_info['Key'];
                    if ($file_info['type'] == 'f')
                        echo "<li class=\"$ext\"><a id=\"" . uniqid() . "\" href=\"#\" rel=\"" . $furl . "\">" . $nprefix . "</a></li>";
                    else
                        echo "<li class=\"directory collapsed\"><a id=\"" . uniqid() . "\" href=\"#\"  rel='$bucket|{$prefix}{$nprefix}/' data-bucket='{$bucket}' data-prefix=\"{$prefix}{$nprefix}\">" . $nprefix . "</a></li>";
                }


            }


            echo "</ul>";
            die();

        }

        function CreateBucket()
        {
            try {
                $this->s3Client->createBucket(array('Bucket' => $_POST['bucketname']));
                echo "Done!";
            } catch (S3Exception $e) {
                echo $e->getMessage();
            }
            die();
        }

        function upload()
        {

            if (!current_user_can(WPDM_ADMIN_CAP)) wp_send_json(array('success' => false, 'message' => __("Unauthorized access!", "download-manager")));

            $file = $_FILES['aws-async-upload']['tmp_name'];
            $filename = $_FILES['aws-async-upload']['name'];

            $bucket = \WPDM\__\Session::get('current_bucket');
            $this->s3Client->putObject(array('Bucket' => trim($bucket, "/"), 'Key' => wpdm_query_var('folder') . '/' . $filename, 'SourceFile' => $file));

            @unlink($file);

            die("ok|uploaded|$bucket|$filename|$file");
        }

        function download($package)
        {

            global $current_user;
            $uid = isset($current_user->ID) ? $current_user->ID : 0;

            //$access = maybe_unserialize(get_post_meta($package['ID'], '__wpdm_access', true));

            //$s3 = new S3(get_option('_wpdm_aws_access_key'), get_option('_wpdm_aws_secret_key'));
            $files = maybe_unserialize($package['files']);

            //if (!isset($_GET['ind']) && count($files) > 1) return $package;

            //$ind = $_GET['ind']?intval($_GET['ind']):0;
            $file = isset($_GET['ind'], $files[$_GET['ind']]) ? $files[$_GET['ind']] : array_shift($files);
            if (substr_count($file, 's3.amazonaws.com/') > 0) {
                $parts = explode(".s3.amazonaws.com/", str_replace(array("http://", "https://"), "", $file));

                $bucket = $parts[0];
                $uri = $parts[1];
                $lifetime = 3600;
                $tmpurl = $this->GetAuthenticatedURL($bucket, $uri, $lifetime);

                WPDM()->downloadHistory->add($package['ID'], $file, wpdm_query_var('oid'));

                header("location: " . $tmpurl);
                die();
            }
        }

        function downloadFile($file)
        {

            if (!strpos($file, 's3.amazonaws.com/')) return true;
            $parts = explode(".s3.amazonaws.com/", str_replace(array("http://", "https://"), "", $file));

            $bucket = $parts[0];
            $uri = $parts[1];
            $lifetime = 3600;
            $tmpurl = $this->GetAuthenticatedURL($bucket, $uri, $lifetime);

            header("location: " . $tmpurl);
            die();
        }

        /**
         * Get a query string authenticated URL
         *
         * @param string $bucket Bucket name
         * @param string $uri Object URI
         * @param integer $lifetime Lifetime in seconds
         * @param boolean $hostBucket Use the bucket name as the hostname
         * @param boolean $https Use HTTPS ($hostBucket should be false for SSL verification)
         * @return string
         */
        function GetAuthenticatedURL($bucket, $uri, $lifetime, $hostBucket = false, $https = true)
        {

            $cmd = $this->s3Client->getCommand('GetObject', [
                'Bucket' => $bucket,
                'Key' => $uri
            ]);

            $request = $this->s3Client->createPresignedRequest($cmd, '+20 minutes');

            // Get the actual presigned-url
            $presignedUrl = (string)$request->getUri();

            return $presignedUrl;

            $expires = time() + $lifetime;
            //$access = get_option('_wpdm_aws_access_key');
            $access = self::getS3Cred('access');
            if (!$access) return null;
            //$access = $key['access']; //get_option('_wpdm_aws_access_key');
            $uri = str_replace(array('%2F', '%2B'), array('/', '+'), rawurlencode($uri));
            return sprintf(($https ? 'https' : 'http') . '://%s/%s?AWSAccessKeyId=%s&Expires=%u&Signature=%s',
                // $hostBucket ? $bucket : $bucket.'.s3.amazonaws.com', $uri, self::$__accessKey, $expires,
                $hostBucket ? $bucket : $bucket . '.s3.amazonaws.com', $uri, $access, $expires,
                urlencode(self::__getHash("GET\n\n\n{$expires}\n/{$bucket}/{$uri}")));
        }

        /**
         * Creates a HMAC-SHA1 hash
         *
         * This uses the hash extension if loaded
         *
         * @param string $string String to sign
         * @return string
         * @internal Used by __getSignature()
         */
        private static function __getHash($string)
        {
            //$privatekey = get_option('_wpdm_aws_secret_key');
            //$access = get_option('_wpdm_aws_access_key');
            $privatekey = self::getS3Cred('secret');
            if (!$privatekey) return null;
            //$privatekey = $key['private'];
            return base64_encode(extension_loaded('hash') ?
                hash_hmac('sha1', $string, $privatekey, true) : pack('H*', sha1(
                    (str_pad($privatekey, 64, chr(0x00)) ^ (str_repeat(chr(0x5c), 64))) .
                    pack('H*', sha1((str_pad($privatekey, 64, chr(0x00)) ^
                            (str_repeat(chr(0x36), 64))) . $string)))));
        }

        function MetaboxHTML($post)
        {

            ?>


            <div class="w3eden inside">
                <?php if (self::getS3Cred('secret')): ?>

                    <div class="row" style="margin-top: 10px">
                        <div class="col-md-6" style="padding-right: 0">
                            <button type="button" class="btn btn-success btn-sm btn-block" onclick="ccreateBucket()"><i
                                        class="fa fa-plus-circle"></i> Add Bucket
                            </button>&nbsp; &nbsp;
                        </div>
                        <div id="aws-plupload-upload-ui" class="hide-if-no-js col-md-6">
                            <div id="aws-drag-drop-area">
                                <div class="aws-drag-drop-inside">
                                    <button id="aws-plupload-browse-button" type="button"
                                            class="btn btn-primary btn-sm btn-block"><i
                                                class="fa fa-upload"></i> <?php esc_attr_e('Upload'); ?></button>
                                </div>
                            </div>
                        </div>
                    </div>

                    <div style="position: absolute;top: -37px;right: 20px;color: var(--color-red);">
                        <i class="fa fa-sun fa-spin" id="awloading" style="display: none"></i>
                    </div>
                    <div id="awstree" style="height: 200px;overflow: auto;"></div>
                    <script>
                        if (jQuery) (function ($) {

                            $.extend($.fn, {
                                awsTree: function (o, h) {
                                    // Defaults
                                    if (!o) var o = {};
                                    if (o.root == undefined) o.root = '/';
                                    if (o.script == undefined) o.script = 'jqueryFileTree.php';
                                    if (o.folderEvent == undefined) o.folderEvent = 'click';
                                    if (o.expandSpeed == undefined) o.expandSpeed = 500;
                                    if (o.collapseSpeed == undefined) o.collapseSpeed = 500;
                                    if (o.expandEasing == undefined) o.expandEasing = null;
                                    if (o.collapseEasing == undefined) o.collapseEasing = null;
                                    if (o.multiFolder == undefined) o.multiFolder = true;
                                    if (o.loadMessage == undefined) o.loadMessage = 'Loading...';

                                    $(this).each(function () {

                                        function showTree(c, t) {
                                            $(c).addClass('wait');
                                            $(".jqueryFileTree.start").remove();
                                            $.post(o.script, {dir: t}, function (data) {
                                                $(c).find('.start').html('');
                                                $(c).removeClass('wait').append(data);
                                                if (o.root == t) $(c).find('UL:hidden').show(); else $(c).find('UL:hidden').slideDown({
                                                    duration: o.expandSpeed,
                                                    easing: o.expandEasing
                                                });
                                                bindTree(c);
                                            });
                                        }

                                        function bindTree(t) {
                                            $(t).find('LI A').bind(o.folderEvent, function () {
                                                if ($(this).parent().hasClass('directory')) {
                                                    if ($(this).parent().hasClass('collapsed')) {
                                                        // Expand
                                                        if (!o.multiFolder) {
                                                            $(this).parent().parent().find('UL').slideUp({
                                                                duration: o.collapseSpeed,
                                                                easing: o.collapseEasing
                                                            });
                                                            $(this).parent().parent().find('LI.directory').removeClass('expanded').addClass('collapsed');
                                                        }
                                                        $(this).parent().find('UL').remove(); // cleanup
                                                        showTree($(this).parent(), escape($(this).attr('rel').match(/.*\//)));
                                                        $(this).parent().removeClass('collapsed').addClass('expanded');
                                                    } else {
                                                        // Collapse
                                                        $(this).parent().find('UL').slideUp({
                                                            duration: o.collapseSpeed,
                                                            easing: o.collapseEasing
                                                        });
                                                        $(this).parent().removeClass('expanded').addClass('collapsed');
                                                    }
                                                    current_bucket = $(this).data('bucket') !== undefined ? $(this).data('bucket') : '';
                                                    current_folder = $(this).data('prefix') !== undefined ? $(this).data('prefix') : '';
                                                } else {
                                                    h($(this).attr('rel'), $(this).attr('id'));
                                                }
                                                /*alert($(this).attr('rel'));*/
                                                return false;
                                            });
                                            // Prevent A from triggering the # on non-click events
                                            if (o.folderEvent.toLowerCase != 'click') $(t).find('LI A').bind('click', function () {
                                                return false;
                                            });
                                        }

                                        // Loading message
                                        $(this).html('<ul class="jqueryFileTree start"><li class="wait">' + o.loadMessage + '<li></ul>');
                                        // Get the initial file list
                                        showTree($(this), escape(o.root));
                                    });
                                }
                            });

                        })(jQuery);
                    </script>
                <link rel="stylesheet"
                      href="<?php echo plugins_url('/download-manager/assets/css/jqueryFileTree.css'); ?>"/>
                    <script language="JavaScript">

                        function awsfiletree() {
                            jQuery('#awstree').awsTree({
                                root: 'awsbuckets',
                                script: ajaxurl + '?wpdmaws=1&action=ListBuckets',
                                expandSpeed: 1000,
                                collapseSpeed: 1000,
                                multiFolder: false
                            }, function (file, id) {
                                var sfilename = file.split('/');
                                var filename = sfilename[sfilename.length - 1];
                                if (confirm('Add this file?')) {
                                    var ID = id;
                                    <?php
                                    $pfiles = maybe_unserialize(get_post_meta($post->ID, '__wpdm_files', true));
                                    if (!is_array($pfiles)) $files = array();

                                    ?>



                                    <?php if(version_compare(WPDM_Version, '4.0.0', '>')){  ?>
                                    var html = jQuery('#wpdm-file-entry').html();
                                    var ext = file.split('.');
                                    ext = ext[ext.length - 1];
                                    var icon = "<?php echo WPDM_BASE_URL; ?>file-type-icons/48x48/" + ext + ".png";
                                    html = html.replace(/##filepath##/g, file);
                                    html = html.replace(/##filetitle##/g, file);
                                    html = html.replace(/##fileindex##/g, ID);
                                    html = html.replace(/##preview##/g, icon);
                                    jQuery('#currentfiles').prepend(html);
                                    <?php } else { ?>
                                    jQuery('#wpdmfile').val(file); /*+"#"+filename*/
                                    jQuery('#cfl').html('<div><strong>' + filename + '</strong>').slideDown();
                                    <?php } ?>


                                }
                            });
                        }

                        jQuery(function () {

                            awsfiletree();

                        });

                    </script>

                <?php

                $plupload_init = array(
                    'runtimes' => 'html5,silverlight,flash,html4',
                    'browse_button' => 'aws-plupload-browse-button',
                    'container' => 'aws-plupload-upload-ui',
                    'drop_element' => 'aws-drag-drop-area',
                    'file_data_name' => 'aws-async-upload',
                    'multiple_queues' => true,
                    /* 'max_file_size'       => wp_max_upload_size().'b',*/
                    'url' => admin_url('admin-ajax.php'),
                    'flash_swf_url' => includes_url('js/plupload/plupload.flash.swf'),
                    'silverlight_xap_url' => includes_url('js/plupload/plupload.silverlight.xap'),
                    'filters' => array(array('title' => __('Allowed Files'), 'extensions' => '*')),
                    'multipart' => true,
                    'urlstream_upload' => true,

                    // additional post data to send to our ajax hook
                    'multipart_params' => array(
                        '_ajax_nonce' => wp_create_nonce('aws-photo-upload'),
                        'wpdmaws' => 1,
                        'action' => 'wpdm_aws_upload',            // the ajax action name
                    ),
                );

                // we should probably not apply this filter, plugins may expect wp's media uploader...
                $plupload_init = apply_filters('plupload_init', $plupload_init); ?>

                    <script type="text/javascript">
                        var current_bucket = '', current_folder = '';

                        function ccreateBucket() {
                            var bname = prompt("Enter Bucket Name:");
                            if (bname == null || bname == '') return false;
                            jQuery('#awloading').fadeIn();
                            jQuery.post(ajaxurl + '?wpdmaws=1&action=CreateBucket', {bucketname: bname}, function (res) {
                                awsfiletree();
                                jQuery('#awloading').fadeOut();
                            });

                        }

                        jQuery(function ($) {


                            // create the uploader and pass the config from above
                            var uploader = new plupload.Uploader(<?php echo json_encode($plupload_init); ?>);

                            // checks if browser supports drag and drop upload, makes some css adjustments if necessary
                            uploader.bind('Init', function (up) {
                                var uploaddiv = jQuery('#aws-plupload-upload-ui');

                                if (up.features.dragdrop) {
                                    uploaddiv.addClass('drag-drop');
                                    jQuery('#aws-drag-drop-area')
                                        .bind('dragover.wp-uploader', function () {
                                            uploaddiv.addClass('drag-over');
                                        })
                                        .bind('dragleave.wp-uploader, drop.wp-uploader', function () {
                                            uploaddiv.removeClass('drag-over');
                                        });

                                } else {
                                    uploaddiv.removeClass('drag-drop');
                                    jQuery('#aws-drag-drop-area').unbind('.wp-uploader');
                                }
                            });

                            uploader.init();

                            uploader.bind('BeforeUpload', function (up, file) {
                                uploader.settings.multipart_params.bucket = current_bucket;
                                uploader.settings.multipart_params.folder = current_folder;
                            });

                            // a file was added in the queue
                            uploader.bind('FilesAdded', function (up, files) {
                                //var hundredmb = 100 * 1024 * 1024, max = parseInt(up.settings.max_file_size, 10);

                                jQuery('#awloading').fadeIn();


                                plupload.each(files, function (file) {
                                    jQuery('#aws-filelist').html(
                                        '<div class="file" id="' + file.id + '"><b>' +

                                        file.name + '</b> (<span>' + plupload.formatSize(0) + '</span>/' + plupload.formatSize(file.size) + ') ' +
                                        '<div class="fileprogress"></div></div>');
                                });

                                up.refresh();
                                up.start();
                            });

                            uploader.bind('UploadProgress', function (up, file) {

                                jQuery('#' + file.id + " .fileprogress").width(file.percent + "%");
                                jQuery('#' + file.id + " span").html(plupload.formatSize(parseInt(file.size * file.percent / 100)));
                            });


                            // a file was uploaded
                            uploader.bind('FileUploaded', function (up, file, response) {
                                awsfiletree();
                                jQuery('#awloading').fadeOut();
                                // this is your ajax response, update the DOM with it or something...
                                //console.log(response);
                                //response
                                jQuery('#' + file.id).remove();
                                var d = new Date();
                                var ID = d.getTime();
                                response = response.response;
                                var nm = response;
                                //if(response.length>20) nm = response.substring(0,7)+'...'+response.substring(response.length-10);
                                //jQuery('#file_source_url').val(response);


                            });

                        });

                    </script>
                    <div id="aws-filelist"></div>
                <?php else : ?>
                    <div id="aws-filelist">To activate this feature you need to <a
                                href='edit.php?post_type=wpdmpro&page=settings&tab=cloud-storage'>Configure Amazon
                            S3</a></div>
                <?php endif; ?>

                <div class="clear"></div>

                <div class="clear"></div>
            </div>


            <?php

        }

        function Metabox($metaboxes)
        {
            $metaboxes['amazons3'] = array('title' => __('Amazon S3 File Source', 'wpdmpro'), 'callback' => array($this, 'MetaboxHTML'), 'position' => 'side');
            return $metaboxes;

        }


        function settings()
        {

            include __DIR__ . '/tpls/settings.php';

        }

        static function getS3Cred($type = null)
        {
            $_wpdm_aws_keys = get_option('_wpdm_aws_key', false);
            $__wpdm_aws_selected_keys = get_option('__wpdm_aws_selected_keys');
            if (!$_wpdm_aws_keys && !$__wpdm_aws_selected_keys) return false;
            if (!$_wpdm_aws_keys) {
                $__wpdm_aws_selected_keys = round(microtime(true) * 1000);
                $_wpdm_aws_keys = array($__wpdm_aws_selected_keys => array('access' => get_option('_wpdm_aws_access_key'), 'secret' => get_option('_wpdm_aws_secret_key'), 'note' => 'Default'));
            }
            if (!isset($_wpdm_aws_keys[$__wpdm_aws_selected_keys])) {
                $keys = array_keys($_wpdm_aws_keys);
                $__wpdm_aws_selected_keys = $keys[0];
            }

            $keys = $_wpdm_aws_keys[$__wpdm_aws_selected_keys];
            if ($type == null || !isset($keys[$type]))
                return $keys;
            return $keys[$type];

        }

        function deleteFile()
        {
            $file = wpdm_query_var('prefix') === '/' ? '' : wpdm_query_var('prefix');
            $ret = $this->s3Client->deleteObject([
                'Bucket' => wpdm_query_var('bucket'),
                'Key' => $file
            ]);
            wp_send_json(array('success' => true));
        }

        function deletePackageFiles($PID, $POST)
        {
            if(get_post_type($POST) !== 'wpdmpro') return;
            $files = get_post_meta($PID, '__wpdm_files', true);
            $files = maybe_unserialize($files);
            foreach ($files as $file) {
                if(substr_count($file, ".s3.amazonaws.com/")) {
                    $file = str_replace("https://", "", $file);
                    list($bucket, $file) = explode(".s3.amazonaws.com/", $file);
                    $this->s3Client->deleteObject([
                        'Bucket' =>$bucket,
                        'Key' => $file
                    ]);
                }

            }
        }

        function adminHead()
        {
            ?>
            <style>
                #wpdms3 {
                    width: 52px;
                }
                body.modal-open {
                    overflow: hidden;
                    padding-right: 15px !important;
                }
            </style>
            <script>
                jQuery(function ($) {
                    $('.pconfig-s3').on('click', function (e) {
                        e.preventDefault();
                        WPDM.bootAlert("Configure Amazon S3 for <span style='color: var(--color-primary)'>"+$(this).data('title')+"</span>","<div style='text-align: center;padding: 50px'><i class='fa fa-sun fa-spin'></i> Loading Options...</div>", 500);
                        $('#__bootModal .modal-body').load(ajaxurl, {action: 's3PackageOptions', pid: $(this).data('pid')});
                    });
                });
            </script>
            <?php
        }


    }


    new WPDM_AmazonS3();


}
