<?php
/**
 * Plugin Name:  WPDM - Category Manager
 * Plugin URI: https://www.wpdownloadmanager.com/download/wpdm-Category-manager/
 * Description: Allows to manage categories from frontend
 * Author: W3 Eden, Inc.
 * Version: 2.1.0
 * Author URI: https://www.wpdownloadmanager.com/
 */


namespace WPDM\AddOn;


// Exit if accessed directly
use WPDM\__\__;
use WPDM\__\Messages;
use WPDM\Category\CategoryController;

if (!defined('ABSPATH')) {
    exit;
}

global $wpdm_cat_man;

class CatMan {

    function __construct() {
        add_action('wp_ajax_wpdm_list_categories', array($this, 'categoryManager'));
        add_action('wp_ajax_wpdm_cdd', array($this, 'categoryDropDown'));
        add_filter('wpdm_frontend', array($this, 'categoryMenu'));
        add_filter('wpdm_settings_frontend', array($this, 'settings'));
        add_shortcode('wpdm_category_manager', array($this, 'catManShortcode'));
        add_filter('WPDM_Category_CategoryController_hArray', array($this, 'allowedCategories'), 10, 2);
    }

    function categoryMenu($menus){
        if(!get_option('__wpdm_enable_category_manager', 0)) return $menus;
        $cmenu = array("categories" => array('label'=>'Categories','callback'=>array($this, 'manageCategories')));
        __::array_splice_assoc($menus, 2, 0, $cmenu);
        return $menus;
    }

    function manageCategories(){
        if(get_option('__wpdm_enable_category_manager', 0))
            include(wpdm_tpl_path('manage-categories.php', __DIR__.'/tpls'));

    }

    function userRoot(){
        global $current_user;
        $user_home = 0;
        if(get_option('__wpdm_enable_user_root_cat',0) == 1) {
            $user_home = get_user_meta($current_user->ID, '__wpdmfb_root_cat', true);
            if (!$user_home || !term_exists($user_home, 'wpdmcategory')) {
                $term = $this->createCategory('Root Category for ' . $current_user->user_login, 0);
                if (!is_wp_error($term) && isset($term['term_id'])) {
                    update_user_meta($current_user->ID, '__wpdmfb_root_cat', $term['term_id']);
                    $user_home = $term['term_id'];
                }
            }
        }
        return $user_home;
    }

    function catManShortcode(){
        ob_start();
        $this->manageCategories();
        return ob_get_clean();
    }

    function categoryManager($params = array()){
        global $current_user, $wp_query;

        if(!get_option('__wpdm_enable_category_manager', 0)){
            Messages::error(__("Category manager is not active","wpdmpro"), 1);
        }

        if(!is_user_logged_in()){
            echo WPDM()->user->login->form($params);
            die();
        }

        //$this->DeleteFolder();
        //$this->DeleteFile();

        $limit = 20;
        $parent = wpdm_query_var('catg', 'int') > 0 ? wpdm_query_var('catg', 'int') : $this->userRoot();

        if(wpdm_query_var('dcat') > 0) {
            wp_delete_term(wpdm_query_var('dcat'), 'wpdmcategory');
            die("OK");
        }

        if(wpdm_query_var('_wpdmtask') == 'createCategory') {
            $slug = sanitize_title($_POST['cname']);
            if (term_exists($_POST['cname'], 'wpdmcategory'))
                $slug = $slug . "-" . uniqid();
            $term = $this->createCategory($_POST['cname'], $parent, $slug);
        }

        if(wpdm_query_var('_wpdmtask') == 'updateCategory') {
            $slug = sanitize_title($_POST['cname']);
            $_parent = $_POST['parent'] > 0?$_POST['parent'] :0;
            //categoryParents($_parent)
            $parents = WPDM()->categories::categoryParents($_parent);

            if($_parent == (int)$_POST['id'] || in_array((int)$_POST['id'], $parents)) $_parent = 0;

            if($_parent == 0)
                $_parent = $this->userRoot();
            wp_update_term((int)$_POST['id'], 'wpdmcategory', array(
                'name' => $_POST['cname'],
                'slug' => $slug,
                'parent' => $_parent
            ));
        }


        $cats = get_terms('wpdmcategory', array('hide_empty' => false, 'parent' => $parent));

        $cond[] = "uid='{$current_user->ID}'";
        $Q = wpdm_query_var('q','txt');
        $paged = $wp_query->query_vars['paged']?$wp_query->query_vars['paged']:1;

        $start = $paged?(($paged-1)*$limit):0;
        $field = wpdm_query_var('sfield')?wpdm_query_var('sfield'):'publish_date';
        $ord = wpdm_query_var('sorder')?wpdm_query_var('sorder'):'desc';

        $author = $current_user->ID;
        $params = array('post_status'=>array('publish','pending','draft'), 'post_type'=>'wpdmpro', 'author'=> $author, 'offset'=>$start);
        $params['orderby'] = $field;
        $params['order'] = $ord;

        $params['tax_query'] = array(
            array(
                'taxonomy' => 'wpdmcategory',
                'field'    => 'id',
                'terms'    => $parent,
                'include_children' => false
            )
        );

        if($field=='download_count'){
            $params['orderby'] = 'meta_value_num';
            $params['meta_key'] = '__wpdm_download_count';
            $params['order'] = $ord;
        }

        ob_start();
        include(wpdm_tpl_path("list-categories.php", __DIR__.'/tpls'));
        $cats = ob_get_clean();
        ob_start();
        CategoryController::CategoryBreadcrumb($_POST['catg'], 0);
        $bcrumb = ob_get_clean();
        wp_send_json(['cats' => $cats, 'crumb' => $bcrumb]);
    }

    public function createCategory($name, $parent, $slug = ''){
        $slug = $slug?$slug:$name;
        if($term = term_exists($slug, 'wpdmcategory', $parent)) return $term;
        $ret =  wp_insert_term(
            $name,
            'wpdmcategory',
            array(
                'parent'=> $parent,
                'slug' => $slug
            )
        );

        if(is_array($ret) && isset($ret['term_id'])) {
            add_user_meta(get_current_user_id(), '__wpdm_mycats', $ret['term_id']);
            return $ret;
        }
        return false;
    }

    function categoryDropDown(){
        wp_dropdown_categories( array('show_option_none' => __( 'Select Parent','wpdmpro' ), 'hide_empty' => 0, 'class' => 'form-control', 'taxonomy' => 'wpdmcategory', 'name' => 'uparent', 'hierarchical' => 1, 'depth' => 10, 'child_of' => $this->userRoot()) );
        die();
    }

    function settings(){
        include __DIR__.'/tpls/catman-settings.php';
    }

    function allowedCategories($categories){

        global $current_user;
        $opt = get_option('__wpdm_artap');
        $cats = array();
        $_html = "<ul>";
        foreach ($current_user->roles as $role){
            $sr = (isset($opt[$role]) && is_array($opt[$role]))?$opt[$role]:array();
            foreach ($sr as $cid){
                $term = get_term($cid, 'wpdmcategory');
                if(!is_wp_error($term) && is_object($term))
                    $cats[$cid] =  array('category' => $term, 'access' => maybe_unserialize(get_term_meta($cid, '__wpdm_access')), 'childs' => array());
            }
            //$cats = $cats + $sr;
        }
        $categories = count($cats) > 0 ? $cats : $categories;
        return  $categories;
    }



}

$wpdm_cat_man = new CatMan();
