<?php
/*
Plugin Name: WPDM - Download Limits
Description: Enable WordPress Download Manager to set download limit for any specified day(s) for user roles
Plugin URI: https://www.wpdownloadmanager.com/download/download-limit/
Author: W3 Eden, Inc.
Version: 2.6.1
Author URI: https://www.wpdownloadmanager.com/
*/

namespace WPDM\AddOn;

global $wpdm_download_limit;

use WPDM\__\Messages;

class DownloadLimit{

    function __construct()
    {
        add_filter("manage_users_columns", array($this, 'userTableColumns'),10, 1);
        add_filter("manage_users_custom_column", array($this, 'userTableCustomData'),10, 3);

        add_filter("wpdm_user_download_limit_exceeded", array($this, 'checkLimit'),10, 2);
        add_filter('wpdm_after_prepare_package_data', array($this, 'downloadLink'));
        add_filter('wdm_before_fetch_template', array($this, 'downloadLink'), 9999);


        add_action('wpdm_onstart_download', array($this, 'updateUserDownload'), 9999);
        add_action("wp_ajax_wpdm_reset_user_download_limit", array($this, 'resetLimit'));
        add_action('basic_settings_section', array($this, 'globalLimitByRoles'));
        add_action('wpdm_is_single_file_download_allowed', array($this, 'singleFileDownload'), 10, 2);
        add_action("admin_footer", array($this, 'adminFooter'));

        add_shortcode("wpdm_user_download_limit", array($this, '_downloadLimit'));
        add_shortcode("wpdm_user_download_count", array($this, 'downloadCount'));
        add_shortcode("wpdm_download_limit_reset_timer", array($this, 'downloadLimitResetTimer'));

        //add_action("wp_footer", [$this, 'showDownloadLimit']);

    }

    function resetLimit(){
        if(isset($_REQUEST['user'])) {
            if($_REQUEST['user'] === 'all') {
                global $wpdb;
                delete_metadata('user', 0, 'wpdm_dd_user_dlc', '', true );
                delete_metadata('user', 0, 'wpdm_reset_time', '', true );
                $wpdb->query("delete from {$wpdb->prefix}options where option_name like 'wpdm_reset_time_%'");
                $wpdb->query("delete from {$wpdb->prefix}options where option_name like 'wpdm_dd_user_dlc_%'");
            }
            else {
                delete_user_meta((int)$_REQUEST['user'], 'wpdm_dd_user_dlc');
                delete_user_meta((int)$_REQUEST['user'], 'wpdm_reset_time');
            }
            wp_send_json(array('success' => 1));
        }
    }

    function validateResetTimer(){

        global $current_user;

        if(is_user_logged_in()) {
            $time = get_user_meta($current_user->ID, 'wpdm_reset_time', true);

            if ($time < time()) {
                update_user_meta($current_user->ID, 'wpdm_reset_time', strtotime("+" . get_option("_wpdm_ddl_period", 1) . " day"));
                update_user_meta($current_user->ID, 'wpdm_dd_user_dlc', 0);
            }
        } else {
            $ipkey = 'wpdm_reset_time_'.str_replace(":","_",$_SERVER['REMOTE_ADDR']);

            $time = get_option('wpdm_reset_time_'.str_replace(":","_",$_SERVER['REMOTE_ADDR']), 0);
            if( $time < time() ) {
                update_option('wpdm_reset_time_'.str_replace(":","_",$_SERVER['REMOTE_ADDR']),strtotime("+".get_option("_wpdm_ddl_period",1)." day"), false);
                //$gkey = 'wpdm_dd_user_dlc_'.str_replace(":","_",$_SERVER['REMOTE_ADDR']);
                //update_option($gkey,0, false);
            }
        }
    }

    /**
     * Check user download limit
     * @param $ID - Package ID
     * @return bool
     */
    function withinLimit($ID){
        global $current_user;
        //With the MasterKey, you are always in limit
        if(WPDM()->package->validateMasterKey($ID, wpdm_query_var('masterkey'))) return true;

        //If you purchased, you have unlimted access
        if(class_exists('\WPDMPP\WPDMPremiumPackage') && \WPDMPP\WPDMPremiumPackage::hasPurchased($ID, get_current_user_id())) return true;

        $this->validateResetTimer();

        $user_download_count = $this->getUserDownloadCount();

        $user_download_limit = $this->getUserDownloadLimit();

        if((int)$user_download_limit ===  0) return true; // Unlimited - No limit

        if($user_download_count > $user_download_limit) return false;

        return true;
    }

    function limitExceededMessage(){
        $msg = get_option('__wpdm_download_limit_exceeded', '');
        if($msg === '') $msg = __('Download Limit Exceeded!', 'wpdm-daily-downloads');
        $msg = stripslashes_deep($msg);
        $msg = wpdm_escs(htmlspecialchars_decode($msg));
        $msg = do_shortcode($msg);
        return $msg;
    }

    function getUserDownloadCount($ID = null){
        if(!is_user_logged_in() && !$ID){
            $gkey = 'wpdm_dd_user_dlc_'.str_replace(":","_",$_SERVER['REMOTE_ADDR']);
            $user_download_count = (int)get_option($gkey,0);
        } else {
            if(!$ID) $ID = get_current_user_id();
            $user_download_count = (int)get_user_meta($ID, 'wpdm_dd_user_dlc', true);
        }

        return $user_download_count;

    }

    function getUserDownloadLimit($ID = null){
        global  $current_user;

        $user = ($ID > 0) ? get_user_by('id', $ID) : $current_user;

        $global_download_limit =  0;
        $global_role_ddl = get_option('_wpdm_global_role_limit',array());

        $roles = is_user_logged_in()?$user->roles :array('guest');
        $role_download_limit = 0;

        foreach ($roles as $role) {
            $global_role_ddl[$role] = isset($global_role_ddl[$role]) ? $global_role_ddl[$role] : 0;
            $role_download_limit = $role_download_limit + (int)$global_role_ddl[$role];
        }

        $user_download_limit = $role_download_limit?$role_download_limit:$global_download_limit;

        return $user_download_limit;
    }

    function globalLimitByRoles(){
        global $current_user;

        $global_role_ddl = get_option('_wpdm_global_role_limit',array());


        ?>
        <div class="panel panel-default">
            <div class="panel-heading"><?php _e('Download Limit', 'wpdm-daily-download'); ?></div>
            <table class="table">
                <tr><td><?php _e('Limit Reset Period:', 'wpdm-daily-download'); ?></td><td style="width: 150px;text-align: right">
                        <div class="input-group">
                            <input type="number" name="_wpdm_ddl_period" class="form-control input-sm" value="<?php echo get_option('_wpdm_ddl_period',1); ?>" />
                            <div class="input-group-addon"> <?php _e('day(s)', 'wpdm-daily-download'); ?> </div>
                        </div>

                    </td></tr>

                <tr><th colspan="2"><?php _e('Limit Download By User Roles', 'wpdm-daily-download'); ?></th> </tr>
                <tr><th><?php _e('User Role', 'wpdm-daily-download'); ?></th><th><?php _e('Download Limit', 'wpdm-daily-download'); ?></th></tr>
                <tr><td>Guests</td><td style="width: 100px;text-align: right"><input  class="form-control input-sm" style="width: 100px" type="text" name="_wpdm_global_role_limit[guest]" value="<?php echo isset($global_role_ddl['guest'])?$global_role_ddl['guest']:""; ?>" /></td></tr>
                <?php
                global $wp_roles;
                $roles = array_reverse($wp_roles->role_names);
                foreach( $roles as $role => $name ) {
                    ?>
                    <tr><td><?php echo $name; ?></td><td style="width: 100px;text-align: right"><input  class="form-control input-sm" style="width: 100px" type="text" size="5" name="_wpdm_global_role_limit[<?php echo $role; ?>]" value="<?php echo isset($global_role_ddl[$role])?$global_role_ddl[$role]:""; ?>"></td></tr>
                <?php } ?>
            </table>
            <div class="panel-footer text-right">
                <button type="button" style="min-width: 230px" class="btn btn-danger" id="__rle"><?php _e('Reset limit for everyone', 'wpdm-daily-download'); ?></button>
            </div>

        </div>

        <?php
    }

    function updateUserDownload($package){
        global $current_user, $wpdmpp;


        if(wpdm_query_var('masterkey') !== '' && WPDM()->package->validateMasterKey($package['ID'], wpdm_query_var('masterkey'))) return;

        if(class_exists('\WPDMPP\WPDMPremiumPackage') && \WPDMPP\WPDMPremiumPackage::hasPurchased($package['ID'], get_current_user_id())) return;


        $user_download_count = $this->getUserDownloadCount();

        $user_download_limit = $this->getUserDownloadLimit();

        if($user_download_count >= $user_download_limit && $user_download_limit > 0) Messages::message($this->limitExceededMessage(), 1);

        $user_download_count++;

        if(is_user_logged_in()){
            update_user_meta($current_user->ID,'wpdm_dd_user_dlc',$user_download_count);
        } else {
            $gkey = 'wpdm_dd_user_dlc_'.str_replace(":","_",$_SERVER['REMOTE_ADDR']);
            update_option($gkey,$user_download_count, false);
        }

        //Initiate timer on the first download
        $this->validateResetTimer();

    }

    /**
     * Validate package download button
     * @param $package
     * @return mixed
     */
    function downloadLink($package){
        if(!$this->withinLimit($package['ID'])){
            $package['download_link'] =
            $package['download_link_extended'] =
            $package['download_link_popup'] = $this->limitExceededMessage();

        }
        return $package;
    }

    /**
     * Validate single file download button
     * @param $allowed
     * @param $ID
     * @return int
     */
    function singleFileDownload($allowed, $ID){
        if(!$this->withinLimit($ID)) {
            $allowed = 0;
        }
        return $allowed;
    }

    function adminFooter(){
        ?>
        <script>
            jQuery(function ($) {
                $('.wpdm-reset-limit').on('click', function () {
                    $(this).html('Reseting...');
                    var $this = $(this);
                    $.post(ajaxurl, {action: 'wpdm_reset_user_download_limit', user: $(this).data('user')}, function () {
                        $this.html('<span style="color: #39bd62"><i class="fa fa-check-double"></i> Done</span>');
                    });
                });

                $('body').on('click', '#__rle', function () {
                    $(this).attr('disabled', 'disabled').html('<i class="fa fa-sun fa-spin"></i>');
                    var $this = $(this);
                    $.post(ajaxurl, {action: 'wpdm_reset_user_download_limit', user: 'all'}, function () {
                        $this.html('<i class="fa fa-check-double"></i> Done');
                    });
                });
            });
        </script>
        <?php
    }

    function userTableColumns( $columns ){
        $columns['downloadlimit'] = 'Download Limit';
        return $columns;
    }

    function userTableCustomData( $val, $column_name, $user_id ) {
        switch ($column_name) {
            case 'downloadlimit' :
                $user_download_limit = $this->_downloadLimit(array('user' => $user_id));
                $user_download_limit = $user_download_limit === 0 ? 'Unlimited' : $user_download_limit;
                return $user_download_limit." / ".$this->downloadCount(array('user' => $user_id)). "  [ <a href='#' class='wpdm-reset-limit' data-user='{$user_id}'>Reset</a> ]";
                break;
            default:
        }
        return $val;
    }

    function checkLimit($limitExeeded, $packageID)
    {
        global $current_user;

        $this->validateResetTimer();

        if(!$this->withinLimit($packageID)) {
            $limitExeeded = true;
        }
        return $limitExeeded;
    }

    function _downloadLimit($params = array()){
        global $current_user;
        $user_id = isset($params['user'])?(int)$params['user']:get_current_user_id();

        $user_download_limit = $this->getUserDownloadLimit($user_id);
        $user_download_count = $this->getUserDownloadCount($user_id);

        if($user_download_limit === 0) return __('Unlimited', 'wpdm-daily-download');

        if(isset($params['remains']) && (int)$params['remains'] === 1) {
            return (int)$user_download_limit - (int)$user_download_count;
        }
        return $user_download_limit;

    }

    function downloadCount($params = array()){
        global $current_user;
        $user = isset($params['user'])?(int)$params['user']:get_current_user_id();
        $user_download_count = $this->getUserDownloadCount($user);
        return $user_download_count;

    }

    function downloadLimitResetTimer(){
        global $current_user;
        $this->validateResetTimer();
        if(is_user_logged_in())
            $time = get_user_meta(get_current_user_id(), 'wpdm_reset_time', true);
        else
            $time = get_option('wpdm_reset_time_'.str_replace(":","_",$_SERVER['REMOTE_ADDR']), 0);
        ob_start();
        ?>
        <span id="wpdm-download-limit-reset-timer" style="display: inline-block">

            <span id="wpdm-dl-timer"></span>

        <script>
            /* Set the date we're counting down to */
            var countDownDate = <?php echo $time * 1000; ?>;

            /* Update the count down every 1 second */
            var x = setInterval(function () {


                var now = new Date().getTime();

                var distance = countDownDate - now;

                var days = Math.floor(distance / (1000 * 60 * 60 * 24));
                if ((days + "").length < 2) days = "0" + days;
                var hours = Math.floor((distance % (1000 * 60 * 60 * 24)) / (1000 * 60 * 60));
                if ((hours + "").length < 2) hours = "0" + hours;
                var minutes = Math.floor((distance % (1000 * 60 * 60)) / (1000 * 60));
                if ((minutes + "").length < 2) minutes = "0" + minutes;
                var seconds = Math.floor((distance % (1000 * 60)) / 1000);
                if ((seconds + "").length < 2) seconds = "0" + seconds;

                document.getElementById("wpdm-dl-timer").innerHTML = "<span class='label label-warning'>" + days + " days</span> <span class='label label-primary'>" + hours + " hours</span> <span class='label label-info'>"
                    + minutes + " minutes</span> <span class='label label-success'>" + seconds + " seconds</span>";

                if (distance < 0) {
                    clearInterval(x);
                    document.getElementById("wpdm-dl-timer").innerHTML = "&mdash;";
                }
            }, 1000);
        </script>
        </span>
        <?php
        return ob_get_clean();
    }

    function showDownloadLimit()
    {
        $user_download_count = $this->getUserDownloadCount();
        $user_download_limit = $this->getUserDownloadLimit();
        $percent = (int)(($user_download_count / $user_download_limit) * 100);
        ?>
        <script>
            jQuery(function ($){
                WPDM.floatify("<div class='card'><div class='card-header'>Download Limit: <?= $user_download_count ?> / <?= $user_download_limit ?></div><div class='card-body'><div class='progress'><div class='progress-bar' role='progressbar' style='width: <?= $percent ?>%' aria-valuenow='<?= $percent ?>' aria-valuemin='0' aria-valuemax='100'></div></div></div></div>", "bottom-left");
            });
        </script>
        <?php
    }


}
if(defined('WPDM_VERSION'))
    $wpdm_download_limit =  new DownloadLimit();



