<?php
/**
 * Plugin Name: WPDM - Default Values
 * Description: WordPress Download Manager Add-on to setup default values for new packages.
 * Plugin URI: https://www.wpdownloadmanager.com/download/wpdm-default-values/
 * Author: WordPress Download Manager
 * Version: 1.9.7
 * Updated: 2021.06.09
 * Author URI: https://www.wpdownloadmanager.com/
 */

class WPDM_Default_Values {

    function __construct(){
        if( ! defined('WPDM_BASE_URL') ) return;

        add_filter( 'add_wpdm_settings_tab', array($this, 'settings_tab' ) );
        add_filter( 'get_post_metadata', array($this, 'get_post_metadata' ), 10, 5 );
        add_filter( 'wpdm_after_prepare_package_data', array( $this, 'set_defaults' ), 10, 4 );

        //add_filter( 'post_row_actions', array( $this, 'clone_package_action' ), 10, 2 );
        //add_action( 'init', array( $this, 'clone_package' ), 10, 2 );

        add_action( 'wp_ajax_wpdm_bulk_update', array( $this, 'bulk_update' ), 10, 2 );

        add_action('save_post', array($this, 'save_default_values'));
    }

    function save_default_values($post){
        if(!current_user_can('edit_post', $post)) return;
        if (get_post_type() != 'wpdmpro' || !isset($_POST['file'])) return;
        $package_settings = $_POST['file'];
    }

    function settings_tab($tabs){
        $tabs['default-values'] = wpdm_create_settings_tab('default-values','Default Values',array($this, 'default_values'), 'fa fa-bolt');
        return $tabs;
    }

    function default_values(){
        if(isset($_POST['task']) && $_POST['task'] == 'wdm_save_settings' && isset($_POST['tab']) && $_POST['tab'] == 'default-settings'){
            update_option('__wpdm_defaults',$_POST['wpdm_defaults']);
            die("Default Values Saved Successfully!");
        }
        $default = get_option('__wpdm_defaults', array());

        if( count( $default ) == 0 ){
            $default = array(
                'version'                   => '',
                'link_label'                => '',
                'quota'                     => '',
                'download_limit_per_user'   => '',
                'view_count'                => '',
                'download_count'            => '',
                'package_size'              => '',
                'access'                    => array('guest'),
                'individual_file_download'  => 1,
                'template'                  => '',
                'page_template'             => '',
                'terms_lock'                => '',
                'password_lock'             => '',
                'linkedin_lock'             => '',
                'tweet_lock'                => '',
                'gplusone_lock'             => '',
                'facebooklike_lock'         => '',
                'email_lock'                => '',
                'icon'                      => '',
            );
        }

        include(__DIR__.'/package-settings.php');
    }

    function set_defaults($pack){
        $default = get_option('__wpdm_defaults', array());
        if(!isset($pack['page_template']) || $pack['page_template'] == '')
            $pack['page_template'] = wpdm_valueof($default, 'page_template');
        if(!isset($pack['template']) || $pack['template'] == '')
            $pack['template'] = wpdm_valueof($default, 'template');
        return $pack;

    }

    function bulk_update(){

        if( ! current_user_can(WPDM_ADMIN_CAP)) return;

        if( is_array( $_POST['meta_value'] ) )
            $meta_value = $_POST['meta_value'];
        else
            $meta_value = esc_attr($_POST['meta_value']);

        $posts = get_posts( array('numberposts' => -1, 'post_type' => 'wpdmpro') );

        if(is_array($_POST['meta_name'])){
            foreach ($posts as $post) {
                foreach ($_POST['meta_name'] as $index => $meta_name) {
                    $value = $_POST['meta_value'][$index];
                    if($meta_name === '__wpdm_custom_form_field_name'){
                        $meta_name = '__wpdm_custom_form_field';
                        $value = array('name' => $value);
                    }
                    update_post_meta($post->ID, $meta_name, $value);
                }
            }
            die('ok');
        }

        $meta_name = esc_attr($_POST['meta_name']);

        foreach ($posts as $post){
            if(!add_post_meta($post->ID, $meta_name, $meta_value, true))
              update_post_meta($post->ID, $meta_name, $meta_value);
        }
        die('ok');
    }

    /**
     * Applying default values when creating new package.
     *
     * @since 1.0.0
     *
     * @param null|array|string $check    The value get_metadata() should return a single metadata value, or an
     *                                    array of values.
     * @param int               $post_id  Post ID.
     * @param string            $meta_key Meta key.
     * @param string|array      $single   Meta value, or an array of values.
     * @return array|null|string The attachment metadata value, array of values, or null.
     */

    function get_post_metadata($check, $post_id, $meta_key, $single, $meta_type ){

        global $pagenow;

        if ( ( ( $pagenow == 'post-new.php' || ( isset( $_GET['adb_page'] ) && $_GET['adb_page'] == 'add-new' ) ) && get_post_type() == 'wpdmpro' ) || (substr_count(wpdm_query_var('adb_page'), 'edit-package') > 0 && !isset($_POST['__wpdmepnonce'])) ) {

            $default        = get_option( '__wpdm_defaults', array() );
            $meta_key_tmp   = str_replace("__wpdm_", "", $meta_key );
            // For Allow Access option
            if ( $meta_key_tmp == 'access' ) {
                $default[$meta_key_tmp] = array($default[$meta_key_tmp]);
            }

            // For Email Lock 'Ask Visitor Name' option
            if( $meta_key_tmp == 'custom_form_field' && isset( $default[$meta_key_tmp] ) )
                $default[$meta_key_tmp] = array( $default[$meta_key_tmp] );

            if ( isset( $default[$meta_key_tmp] ) )
                return $default[$meta_key_tmp];
        }

        return $check;
    }

    function clone_package_action($actions, $page_object){
        if( $page_object->post_type != 'wpdmpro' ) return $actions;
        $actions['wpdm_clone'] = '<a href="post.php?post=' . $page_object->ID . '&action=wpdm-clone" class="google_link">' . __('Clone','wpdmpro') . '</a>';
        return $actions;
    }

    function clone_package(){
        if(wpdm_query_var('action','txt') == 'wpdm-clone'){
            $post_data = get_post(wpdm_query_var('post','int'), ARRAY_A);
            unset($post_data['ID']);
            $ID = wp_insert_post($post_data);
            $cdata = get_post_custom(wpdm_query_var('post','int'));

            if(is_array($cdata)){
                foreach ($cdata as $k => $v) {
                    update_post_meta($ID, $k, maybe_unserialize($v[0]));
                }
            }
            wp_redirect("post.php?post={$ID}&action=edit");
            die();
        }
    }

}

new WPDM_Default_Values();



