<?php
/**
 * Display / Edit Billing Info in WordPress's user profile page.
 *
 * @version     1.0.0
 */
namespace WPDMPP\Libs;

// Exit if accessed directly
if (!defined('ABSPATH')) {
    exit;
}

if( ! class_exists( 'BillingInfo' ) ):

    class BillingInfo{

        function __construct()
        {
            add_action( 'show_user_profile',        array( $this, 'wpdmpp_add_billing_info_fields' ) );
            add_action( 'edit_user_profile',        array( $this, 'wpdmpp_add_billing_info_fields' ) );
            add_action( 'personal_options_update',  array( $this, 'wpdmpp_save_billing_info_fields' ) );
            add_action( 'edit_user_profile_update', array( $this, 'wpdmpp_save_billing_info_fields' ) );
        }

        function wpdmpp_add_billing_info_fields($user){
            global $wpdmpp_settings;
            $billing_shipping = unserialize(get_user_meta($user->ID, 'user_billing_shipping',true));
            $billing_defaults =  array
            (
                'first_name'    => '',
                'last_name'     => '',
                'company'       => '',
                'address_1'     => '',
                'address_2'     => '',
                'city'          => '',
                'postcode'      => '',
                'country'       => '',
                'state'         => '',
                'email'         => '',
                'phone'         => '',
                'taxid'         => ''
            );

            //echo '<pre>';print_r($billing_shipping);echo '</pre>';

            if( is_array( $billing_shipping ) && isset( $billing_shipping['billing'] ) )
                $billing = $billing_shipping['billing'];
            else
                $billing = $billing_defaults;

            ?>
            <h2><?php _e('Customer Billing Address','wpdm-premium-packages'); ?></h2>

            <table class="form-table">
                <tr>
                    <th><label for="billing_first_name"><?php _e("Billing First Name", "wpdm-premium-packages"); ?></label></th>
                    <td>
                        <input type="text" value="<?php echo $billing['first_name']; ?>" name="checkout[billing][first_name]" id="billing_first_name" class="regular-text" /><br />
                        <span class="description"><?php _e('Enter your billing first name.', 'wpdm-premium-packages'); ?></span>
                    </td>
                </tr>
                <tr>
                    <th><label for="billing_last_name"><?php _e("Last Name", "wpdm-premium-packages"); ?></label></th>
                    <td>
                        <input type="text" value="<?php echo $billing['last_name']; ?>" placeholder="Last Name" id="billing_last_name" name="checkout[billing][last_name]" class="regular-text"><br />
                        <span class="description"><?php _e('Enter your billing last name.', 'wpdm-premium-packages'); ?></span>
                    </td>
                </tr>
                <tr>
                    <th><label for="billing_company"><?php _e("Company Name", "wpdm-premium-packages"); ?></label></th>
                    <td>
                        <input type="text" value="<?php echo $billing['company']; ?>" placeholder="Company (optional)" id="billing_company" name="checkout[billing][company]" class="regular-text"><br />
                        <span class="description"><?php _e('Enter your company name.', 'wpdm-premium-packages'); ?></span>
                    </td>
                </tr>
                <tr>
                    <th><label class="" for="billing_address_1"><?php _e("Address Line 1", "wpdm-premium-packages"); ?> </label></th>
                    <td>
                        <input type="text" value="<?php echo $billing['address_1']; ?>" placeholder="Address" id="billing_address_1" name="checkout[billing][address_1]" class="regular-text"><br />
                        <span class="description"><?php _e('Enter your billing address line 1.', 'wpdm-premium-packages'); ?></span>
                    </td>
                </tr>
                <tr>
                    <th><label  class="" for="billing_address_2"><?php _e("Address Line 2", "wpdm-premium-packages"); ?></label></th>
                    <td>
                        <input type="text" value="<?php echo $billing['address_2']; ?>" placeholder="Address 2 (optional)" id="billing_address_2" name="checkout[billing][address_2]" class="regular-text"><br />
                        <span class="description"><?php _e('Enter your billing address line 2.', 'wpdm-premium-packages'); ?></span>
                    </td>
                </tr>
                <tr>
                    <th><label class="" for="billing_city"><?php _e("Town/City", "wpdm-premium-packages"); ?> </label></th>
                    <td>
                        <input type="text" value="<?php echo $billing['city']; ?>" placeholder="Town/City" id="billing_city" name="checkout[billing][city]" class="regular-text"><br />
                        <span class="description"><?php _e('Enter your billing city name.', 'wpdm-premium-packages'); ?></span>
                    </td>
                </tr>
                <tr>
                    <th><label class="" for="billing_postcode"><?php _e("Postcode/Zip", "wpdm-premium-packages"); ?> </label></th>
                    <td>
                        <input type="text" value="<?php echo $billing['postcode']; ?>" placeholder="Postcode/Zip" id="billing_postcode" name="checkout[billing][postcode]" class="regular-text"><br />
                        <span class="description"><?php _e('Enter your billing post code.', 'wpdm-premium-packages'); ?></span>
                    </td>
                </tr>
                <tr>
                    <th><label class="" for="billing_country"><?php _e("Country", "wpdm-premium-packages"); ?> </label></th>
                    <td>
                        <?php
                        $countries = wpdmpp_get_countries();
                        $allowed_countries = ( isset($wpdmpp_settings['allow_country'] ) ) ? $wpdmpp_settings['allow_country'] : array();
                        ?>
                        <select class="select" id="billing_country" name="checkout[billing][country]">
                            <option value=""><?php _e('--Select Country--','wpdm-premium-packages'); ?></option>
                            <?php
                            foreach ($countries as $country) {
                                if ( ! empty( $allowed_countries ) ) {
                                    if( in_array( $country->country_code, $allowed_countries ) )
                                        echo '<option value="' . $country->country_code . '"' . selected( $billing['country'], $country->country_code, false ) . '>' . $country->country_name . '</option>';
                                } else {
                                    echo '<option value="' . $country->country_code . '" ' . selected( $billing['country'], $country->country_code, false ) . '>' . $country->country_name . '</option>';
                                }
                            }
                            ?>
                        </select> <br />
                        <span class="description"><?php _e('Enter your billing country name.', 'wpdm-premium-packages'); ?></span>
                    </td>
                </tr>
                <tr>
                    <th><label class="" for="billing_state"><?php _e("State/County", "wpdm-premium-packages"); ?> </label></th>
                    <td>
                        <input type="text" value="<?php echo $billing['state']; ?>" id="billing_state" name="checkout[billing][state]" placeholder="State/County" class="regular-text"><br />
                        <span class="description"><?php _e('Enter your billing state.', 'wpdm-premium-packages'); ?></span>
                    </td>
                </tr>
                <tr>
                    <th><label class="" for="billing_email"><?php _e("Email Address", "wpdm-premium-packages"); ?> </label></th>
                    <td>
                        <input type="text" value="<?php echo $billing['email']; ?>" placeholder="Email Address" id="billing_email" name="checkout[billing][email]" class="regular-text email"><br />
                        <span class="description"><?php _e('Enter your billing email address.', 'wpdm-premium-packages'); ?></span>
                    </td>
                </tr>
                <tr>
                    <th><label class="" for="billing_phone"><?php _e("Phone", "wpdm-premium-packages"); ?> </label></th>
                    <td>
                        <input type="text" value="<?php echo $billing['phone']; ?>" placeholder="Phone" id="billing_phone" name="checkout[billing][phone]" class="regular-text"><br />
                        <span class="description"><?php _e('Enter your billing phone number.', 'wpdm-premium-packages'); ?></span>
                    </td>
                </tr>
                <tr>
                    <th><label class="" for="billing_tin"><?php _e("Tax ID", "wpdm-premium-packages"); ?> </label></th>
                    <td>
                        <input type="text" value="<?php echo $billing['taxid']; ?>" placeholder="Tax Identification Number" id="billing_tin" name="checkout[billing][taxid]" class="regular-text"><br />
                        <span class="description"><?php _e('Enter your Tax Identification Number.', 'wpdm-premium-packages'); ?></span>
                    </td>
                </tr>
            </table>

            <?php
        }

        function wpdmpp_save_billing_info_fields($user_id){
            if ( !current_user_can( 'edit_user', $user_id ) ) return false;
            update_user_meta($user_id, 'user_billing_shipping', serialize($_POST['checkout']));
        }

    }

endif;

new BillingInfo();