<?php
/*
Plugin Name: WPDM - Watermark
Description: Add image watermarks with attached images with custom positioning
Plugin URI: https://www.wpdownloadmanager.com/download/wordpress-image-watermark/
Author: WordPress Download Manager
Version: 1.0.1
Author URI: https://www.wpdownloadmanager.com/
Text Domain: wpdm-watermark
*/

namespace WPDM\AddOn;

include_once "lib/class.ImageWatermark.php";

use WPDM\AddOn\lib\ImageWatermark;
use WPDM\Package;

if(defined('WPDM_Version')) {
    class Watermark
    {
        function __construct()
        {

            register_activation_hook(__FILE__, array($this, 'activate'));
            register_deactivation_hook(__FILE__, array($this, 'deactivate'));

            add_action("wp_footer", array($this, 'watermarkPreview'));
            add_action("admin_init", array($this, 'configure'));
            add_action("wpdm_attached_file", array($this, 'addButton'), 10, 2);
            add_action("wp_ajax_upload_custom_watermark", array($this, 'saveCustomWatermark'));
            add_action("wp_ajax_save_watermark_settings", array($this, 'saveWatermarkSettings'));
            add_action("wp_ajax_wpdm_save_watermark", array($this, 'saveWaterMarkConfig'), 10, 2);
            add_action("wp_ajax_delete_custom_watermark", array($this, 'deleteWatermark'));
            add_action("wpdm_onstart_download", array($this, 'applyWaterMark'), 999999);
            //add_action("wpdm_edit_profile_form", array($this, 'userProfileField'), 0);
            //add_action("wpdm_user_dashboard_menu", array($this, 'dashboardMenu'), 0);
            //add_action("wpdm_update_profile", array($this, 'saveCustomWatermark'), 0);
            add_filter('add_wpdm_settings_tab', array($this, 'settingsTab'));
        }

        function settingsTab($tabs){
            $tabs['wpdm-watermark'] = \WPDM\admin\menus\Settings::createMenu('wpdm-watermark', 'Watermark', array($this, 'settings'), 'fa fa-magic');
            return $tabs;
        }

        function dashboardMenu($menus){
            $menus['wm-logo'] = array('name'=> __( "Set Logo" , "wpdm-watermark" ), 'callback' => array($this, 'setWatermark'));
            return $menus;
        }

        function addButton($packageID, $fileID)
        {
            //$files = Package::getFiles($packageID);
            $file_info = get_post_meta($packageID, '__wpdm_fileinfo', true);
            $file_info = maybe_unserialize($file_info);
            $enabled = (is_array($file_info) && isset($file_info[$fileID], $file_info[$fileID]['watermark'])) ? 1 : 0;
            $url = admin_url("?watermark=config&pid={$packageID}&file={$fileID}");
            echo "<button type='button' onclick='wpdm_iframe_modal(\"{$url}\")' class='btn btn-info' style='margin-top: 10px'>Watermark</button> <label style='margin-top: 10px !important;padding-left: 10px;display: inline-block'><input ".checked(1, $enabled, false)." type='checkbox' name='file[fileinfo][{$fileID}][watermark]' value='1'> ".__('Enable Watermark', 'wpdm-watermark')."</label>";
        }

        function configure()
        {
            if (wpdm_query_var('watermark') === 'config') {
                include dirname(__FILE__) . "/tpls/config.php";
                die();
            }
        }

        function saveWaterMarkConfig()
        {

            if (!wpdm_query_var('pid') || !wpdm_query_var('file')) return;

            $wmConfig = get_post_meta(wpdm_query_var('pid'), '__wm_conf', true);
            $wmConfig = maybe_unserialize($wmConfig);
            $allfiles = Package::getFiles(wpdm_query_var('pid'));
            if (!is_array($wmConfig)) $wmConfig = array();
            $wmConfig[wpdm_query_var('file')] = array('left' => wpdm_query_var('wmpx'), 'top' => wpdm_query_var('wmpy'), 'opacity' => wpdm_query_var('wmopacity'));
            if(wpdm_query_var('applyall', 'num') == 1) {
                foreach ($allfiles as $fileid => $file) {
                    $wmConfig[$fileid] = array('left' => wpdm_query_var('wmpx'), 'top' => wpdm_query_var('wmpy'), 'opacity' => wpdm_query_var('wmopacity'));
                }
            }
            update_post_meta(wpdm_query_var('pid'), '__wm_conf', $wmConfig);
            wp_send_json(array('success' => 1));
        }


        function applyWaterMark($package)
        {
            global $current_user;

            $fileid = wpdm_query_var('ind', 'num');
            $wmfiles = get_post_meta($package['ID'], '__wpdm_wmfiles', true);
            $wmfiles = maybe_unserialize($wmfiles);
            $files = \WPDM\Package::getFiles($package['ID']);
            if(!$fileid && count($files) === 1){
                $fileid = key($files);
            }

            $file_info = get_post_meta($package['ID'], '__wpdm_fileinfo', true);
            $file_info = maybe_unserialize($file_info);
            $enabled = (is_array($file_info) && isset($file_info[$fileid], $file_info[$fileid]['watermark'])) ? 1 : 0;
            if(!$enabled) return;

            $disable_watermark = maybe_unserialize(get_option( '__wpdm_disable_watermark_roles', array('administrator')));
            $roles = is_user_logged_in() ? $current_user->roles : array('guest');
            $disabled = array_intersect($disable_watermark, $roles);
            if($disabled) return;

            if(!is_array($wmfiles) || !isset($wmfiles[$fileid]) || !file_exists($wmfiles[$fileid])) {
                $image = $files[$fileid];
                $ext = explode(".", strtolower($image));
                $ext = end($ext);
                if (!in_array($ext, array('png', 'jpg', 'jpeg')) || Package::validateMasterKey(wpdm_query_var('wpdmdl'), wpdm_query_var('masterkey'))) return;
                $wmConfig = get_post_meta($package['ID'], '__wm_conf', true);
                //wpdmdd($wmConfig);
                $wmleft = isset($wmConfig[$fileid], $wmConfig[$fileid]['left']) ? explode("|", $wmConfig[$fileid]['left']) : array(0);
                $wmtop = isset($wmConfig[$fileid], $wmConfig[$fileid]['top']) ? explode("|", $wmConfig[$fileid]['top']) : array(0);
                $wmopacity = isset($wmConfig[$fileid], $wmConfig[$fileid]['opacity']) ? $wmConfig[$fileid]['opacity'] : 0.50;
                $wmleft = (int)$wmleft[0];
                $wmtop = (int)$wmtop[0];
                $filename = str_replace(".{$ext}", "", strtolower(basename($image)));
                $image = file_exists($image) ? $image : UPLOAD_DIR . $image;
                //wpdmdd($image);
                $imgwm = new ImageWatermark();
                $wmlogo = self::getCustomWatermark();
                //wpdmdd($wmlogo);
                if ($wmlogo) {
                    //$wmlogo = \WPDM\libs\FileSystem::imageThumbnail($wmlogo, 256, 256, true, false);
                    //wpdmdd($filename);
                    $res = $imgwm->stamp($wmlogo)
                        ->image($image)
                        ->left($wmleft)
                        ->top($wmtop)
                        ->opacity($wmopacity)
                        ->filename($filename)
                        ->apply();
                    if(!is_array($wmfiles)) $wmfiles = array();
                    $wmfiles[$fileid] = $res;
                    update_post_meta($package['ID'], '__wpdm_wmfiles', $wmfiles);
                    wpdm_download_file($res, wp_basename($files[$fileid]));
                    die();
                }
            }
            if(is_array($wmfiles) && isset($wmfiles[$fileid]) && file_exists($wmfiles[$fileid])){
                wpdm_download_file($wmfiles[$fileid], wp_basename($files[$fileid]));
                die();
            }
        }


        //function userProfileField()
        function setWatermark()
        {
            ob_start();
            $cwm = get_user_meta(get_current_user_id(), '__wpdm_custom_watermark', true);
            $cwm = str_replace(ABSPATH, home_url('/'), $cwm);
            include_once "tpls/user-profile-field.php";
            return ob_get_clean();
        }

        function settings()
        {
            if(wpdm_query_var('task') === 'wdm_save_settings' && wpdm_query_var('section') === 'wpdm-watermark'){
                update_option('__wpdm_wm_width', $_REQUEST['__wpdm_wm_width'], false);
                update_option('__wpdm_wm_height', $_REQUEST['__wpdm_wm_height'], false);
                if((int)$_REQUEST['__wpdm_wm_width'] > 0)
                    $thumb = wpdm_dynamic_thumb(get_option('__wpdm_watermark'),  array((int)$_REQUEST['__wpdm_wm_width'], (int)$_REQUEST['__wpdm_wm_height']), false, false);
                else
                    $thumb = str_replace(ABSPATH, site_url('/'), get_option('__wpdm_watermark'));
                //$thumb = str_replace(site_url('/'), ABSPATH, $thumb);
                update_option('__wpdm_watermark_resized', $thumb, false);
                update_option('__wpdm_disable_watermark_roles', wpdm_query_var('__wpdm_disable_watermark_roles'), false);
                _e('Watermark saved successfully', 'wpdm-watermark');
                die();
            }
            $cwm = get_option('__wpdm_watermark', '');
            $cwm = str_replace(ABSPATH, home_url('/'), $cwm);
            include_once "tpls/settings.php";
        }

        function saveWatermarkSettings()
        {

            check_ajax_referer(NONCE_KEY);
            if($_FILES['wpdm_file']){
                $type = $_FILES['wpdm_file']['type'];
                if(!in_array($type, array('image/jpeg', 'image/png'))) wp_send_json(array('success' => false, 'msg' => 'Invalid file type'));
                $ext = explode(".", $_FILES['wpdm_file']['name']);
                $ext = end($ext);
                $wmlogo = self::logo_dir().'wpdm-watermark-'.uniqid().'.'.$ext;
                move_uploaded_file($_FILES['wpdm_file']['tmp_name'], $wmlogo);
                $oldlogo = get_option('__wpdm_watermark', '');
                if($oldlogo !== '' && file_exists($oldlogo)) @unlink($oldlogo);
                update_option( '__wpdm_watermark', $wmlogo, false );
                $wmlogo_url = str_replace(ABSPATH, home_url('/'), $wmlogo);
                wp_send_json(array('success' => true, 'image_url' => $wmlogo_url."?refresh=".uniqid()));
            }
        }

        function saveCustomWatermark()
        {

            check_ajax_referer(NONCE_KEY);
            if($_FILES['wpdm_file']){
                $type = $_FILES['wpdm_file']['type'];
                if(!in_array($type, array('image/jpeg', 'image/png', 'image/gif'))) wp_send_json(array('success' => false, 'msg' => 'Invalid file type'));
                $ext = explode(".", $_FILES['wpdm_file']['name']);
                $ext = end($ext);
                $wmlogo = self::logo_dir().'wmlogo-'.get_current_user_id().'.'.$ext;
                move_uploaded_file($_FILES['wpdm_file']['tmp_name'], $wmlogo);
                update_user_meta(get_current_user_id(), '__wpdm_custom_watermark', $wmlogo);
                $wmlogo_url = str_replace(ABSPATH, home_url('/'), $wmlogo);
                wp_send_json(array('success' => true, 'image_url' => $wmlogo_url."?refresh=".uniqid()));
            }
        }

        function deleteWatermark(){
            //delete_user_meta(get_current_user_id(), '__wpdm_custom_watermark');
            delete_option( '__wpdm_watermark');
            wp_send_json(array('success' => true));
        }

        static function getCustomWatermark()
        {
            //if(!is_user_logged_in())
            $cwm = get_option('__wpdm_watermark', ''); //get_user_meta(get_current_user_id(), '__wpdm_custom_watermark', true);
            return $cwm;
        }

        static function activate(){
            if(!file_exists(self::logo_dir())) {
                mkdir(self::logo_dir(), 0755);
                @chmod(self::logo_dir(), 0755);
            }
        }

        static function logo_dir(){
            $upload_dir = wp_upload_dir();
            $upload_dir = $upload_dir['basedir'];
            return $upload_dir.'/wpdm-watermark-logos/';
        }

        function deactivate(){

        }


        public static function fileListExtended($file, $w = 88, $h = 88, $cols = 3)
        {
            global $current_user;

            $package = $file;
            $cwm = get_user_meta(get_current_user_id(), '__wpdm_custom_watermark', true);

            $file['files'] = maybe_unserialize($file['files']);
            $fhtml = '';
            $idvdl = 1; //\WPDM\Package::isSingleFileDownloadAllowed($file['ID']); //isset($file['individual_file_download']) ? $file['individual_file_download'] : 0;
            //$pd = isset($file['publish_date'])&&$file['publish_date']!=""?strtotime($file['publish_date']):0;
            //$xd = isset($file['expire_date'])&&$file['expire_date']!=""?strtotime($file['expire_date']):0;

            $cur = is_user_logged_in()?$current_user->roles:array('guest');

            $permalink = get_permalink($file['ID']);
            $sap = strpos($permalink, '?')?'&':'?';
            $download_url = $permalink.$sap."wpdmdl={$file['ID']}";

            \WPDM\Session::set('wpdmfilelistcd_'.$file['ID'], 1);

            //if(($xd>0 && $xd<time()) || ($pd>0 && $pd>time()))  $idvdl = 0;

            $dir = isset($file['package_dir'])?$file['package_dir']:'';
            $dfiles = array();
            if($dir!=''){
                $dfiles = wpdm_get_files($dir, false);

            }

            $button_label = apply_filters("single_file_download_link_label", __( "Download" , "download-manager" ), $file);

            $file['access'] = wpdm_allowed_roles($file['ID']);

            if (count($file['files']) > 0 || count($dfiles) > 0) {

                $fileinfo = isset($file['fileinfo']) ? $file['fileinfo'] : array();
                $pwdlock = isset($file['password_lock']) ? $file['password_lock'] : 0;

                //Check if any other lock option apllied for this package
                $olock = wpdm_is_locked($file['ID']) ? 1 : 0;

                $swl = 0;
                if(!isset($file['quota'])||$file['quota']<=0) $file['quota'] = 9999999999999;
                if(is_user_logged_in()) $cur[] = 'guest';
                if(!isset($file['access']) || count($file['access'])==0 || !wpdm_user_has_access($file['ID']) || wpdm_is_download_limit_exceed($file['ID']) || $file['quota'] <= $file['download_count']) $olock = 1;
                $pwdcol = $dlcol = '';
                if ($pwdlock && $idvdl) $pwdcol = "<th>".__( "Password" , "download-manager" )."</th>";
                if ($idvdl && ($pwdlock || !$olock)) { $dlcol = "<th align=center>".__( "Download" , "download-manager" )."</th>"; $swl = 1; }
                $allfiles = get_post_meta($file['ID'], '__wpdm_files', true);//is_array($file['files'])?$file['files']:array();


                //$allfiles = array_merge($allfiles, $dfiles);
                $fhtml = "<div class='row' id='xfilelist'>";
                if (is_array($allfiles)) {

                    $classes = array('1' => 'col-md-12', '2' => 'col-md-6', '3' => 'col-md-4', '4' => 'col-md-3', '6' => 'col-md-2');
                    $class = isset($classes[$cols])?$classes[$cols]:'col-md-4';

                    foreach ($allfiles as $fileID => $sfile) {
                        $fhtml .= "<div class='{$class} col-sm-6 col-xs-6'><div class='panel panel-default'>";
                        $ind = $fileID; //\WPDM_Crypt::Encrypt($sfile);

                        if (!isset($fileinfo[$sfile]) || !@is_array($fileinfo[$sfile])) $fileinfo[$sfile] = array();
                        if (!@is_array($fileinfo[$fileID])) $fileinfo[$fileID] = array();

                        $filePass = isset($fileinfo[$sfile]['password'])?$fileinfo[$sfile]['password']:(isset($fileinfo[$fileID]['password'])?$fileinfo[$fileID]['password']:'');
                        $fileTitle = isset($fileinfo[$sfile]['title']) && $fileinfo[$sfile]['title'] != '' ? $fileinfo[$sfile]['title']:(isset($fileinfo[$fileID]['title']) && $fileinfo[$fileID]['title'] != '' ? $fileinfo[$fileID]['title']:preg_replace("/([0-9]+)_/", "",wpdm_basename($sfile)));

                        //$fileTitle = esc_html($fileTitle);

                        if ($filePass == '' && $pwdlock) $filePass = $file['password'];

                        $fhtml .= "<div class='panel-heading ttip' title='{$fileTitle}'>{$fileTitle}</div>";

                        $imgext = array('png','jpg','jpeg', 'gif');
                        $ext = explode(".", $sfile);
                        $ext = end($ext);
                        $ext = strtolower($ext);
                        $filepath = file_exists($sfile)?$sfile:UPLOAD_DIR.$sfile;
                        $thumb = "";

                        if($ext == '' || !file_exists(WPDM_BASE_DIR.'assets/file-type-icons/'.$ext.'.svg')) $ext = 'unknown';

                        if(in_array($ext, $imgext))
                            $thumb = wpdm_dynamic_thumb($filepath, array($w, $h));
                        if($thumb) {
                            $size = getimagesize($filepath);
                            $oimgw = $size[0];
                            $oimgh = $size[1];
                            $wmw = (int)((273/$oimgw)*270);
                            $wmh = (int)((273/$oimgh)*180);
                            $wmConfig = get_post_meta($package['ID'], '__wm_conf', true);
                            $_cwm = wpdm_dynamic_thumb($cwm, array($wmw, $wmh), false);
                            //wpdmdd($wmConfig);
                            $wmleft = isset($wmConfig[$fileID], $wmConfig[$fileID]['left']) ? explode("|", $wmConfig[$fileID]['left']) : array(0);
                            $wmtop = isset($wmConfig[$fileID], $wmConfig[$fileID]['top']) ? explode("|", $wmConfig[$fileID]['top']) : array(0);
                            $wmopacity = isset($wmConfig[$fileID], $wmConfig[$fileID]['opacity']) ? $wmConfig[$fileID]['opacity'] : 0.50;
                            $wmleft = (int)$wmleft[0];
                            $wmtop = (int)$wmtop[0];
                            $wmx = (int)((273/$oimgw)*$wmleft);
                            $wmy = (int)((273/$oimgh)*$wmtop);
                            $fhtml .= "<div class='panel-body text-center'><img class='file-thumb' data-wm='$_cwm' data-wmw='{$wmw}' data-wmh='{$wmh}' data-wmx='$wmx' data-wmy='$wmy' src='{$thumb}' alt='{$fileTitle}' /></div><div class='panel-footer footer-info'>" . wpdm_file_size($sfile) . "</div><div class='panel-footer text-center'>";
                        }
                        else
                            $fhtml .= "<div class='panel-body text-center'><img class='file-ico' src='".WPDM_BASE_URL.'assets/file-type-icons/'.$ext.'.svg'."' alt='{$fileTitle}' /></div><div class='panel-footer footer-info text-center'>".wpdm_file_size($sfile)."</div><div class='panel-footer  text-center'>";


                        if ($swl) {

                            if ($filePass != '' && $pwdlock)
                                $fhtml .= "<div class='input-group input-group-sm'><input  onkeypress='jQuery(this).removeClass(\"input-error\");' size=10 type='password' value='' id='pass_{$file['ID']}_{$ind}' placeholder='Password' name='pass' class='form-control inddlps' />";
                            if ($filePass != '' && $pwdlock)
                                $fhtml .= "<span class='input-group-btn input-group-append'><button class='inddl btn btn-secondary btn-light btn-block' data-pid='{$file['ID']}' data-file='{$fileID}' data-pass='#pass_{$file['ID']}_{$ind}'><i class='fas fa-arrow-alt-circle-down'></i></button></span></div>"; //rel='" . $download_url . "&ind=" . $ind . "'
                            else {
                                $ind_download_link = "<a rel='nofollow' class='inddl btn btn-primary btn-sm' href='" . $download_url . "&ind=" . $ind . "'>".$button_label."</a>";
                                $ind_download_link = apply_filters("wpdm_single_file_download_link", $ind_download_link, $fileID, $package);
                                $individual_file_actions = '';
                                $individual_file_actions = apply_filters("individual_file_action", $individual_file_actions, $file['ID'], $sfile, $fileID);
                                $fhtml .= $ind_download_link."&nbsp;{$individual_file_actions}";
                            }
                        }


                        $fhtml .= "</div></div></div>";
                    }

                }

                $fhtml .= "</div>";
                $siteurl = home_url('/');
                //$fhtml .= "<script type='text/javascript' language='JavaScript'> jQuery('.inddl').click(function(){ var tis = this; jQuery.post('{$siteurl}',{wpdmfileid:'{$file['ID']}',wpdmfile:jQuery(this).attr('file'),actioninddlpvr:1,filepass:jQuery(jQuery(this).attr('pass')).val()},function(res){ res = res.split('|'); var ret = res[1]; if(ret=='error') jQuery(jQuery(tis).attr('pass')).addClass('input-error'); if(ret=='ok') location.href=jQuery(tis).attr('rel')+'&_wpdmkey='+res[2];});}); </script> ";


            }


            return $fhtml;

        }

        function watermarkPreview(){
            if(is_singular('wpdmpro')) {
                ?>

                <script src="https://cdn.jsdelivr.net/npm/watermarkjs@2.0.0/dist/watermark.min.js"></script>
                <script>
                    jQuery(function ($) {
                        $('#xfilelist .file-thumb').each(function (id) {

                            var _img = $(this);

                            var getX = function(boat, logo) {
                                return _img.data('wmx');
                            };

                            var getY = function(boat, logo) {
                                return _img.data('wmy');
                            };

                            watermark([$(this).attr('src'), _img.data('wm')])
                                .image(watermark.image.atPos(getX, getY, 0.9))
                                .then(function (img) {
                                    _img.after(img);
                                    _img.remove();
                                });
                        });
                    });
                </script>

                <?php
            }
        }

        function templateTag($vars, $template, $type){
            preg_match_all("/\[file_list_extended_([0-9]+)x([0-9]+)x([0-9]+)\]/", $template, $flematches);
            //return $vars;
            //Replace all file list extended tags
            foreach ($flematches[0] as $nd => $scode) {
                $scode = str_replace(array("[", "]"), "", $scode);
                $vars[$scode] = self::fileListExtended($vars, $flematches[1][$nd], $flematches[2][$nd], $flematches[3][$nd]);
            }

            return $vars;
        }




    }

    new Watermark();


}
